package cn.com.qmth.scancloud.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.core.exception.ParameterException;

import cn.com.qmth.scancloud.bean.User;
import cn.com.qmth.scancloud.config.SysProperty;
import cn.com.qmth.scancloud.entity.AnswerCardEntity;
import cn.com.qmth.scancloud.entity.ExamEntity;
import cn.com.qmth.scancloud.exception.ParameterExceptions;
import cn.com.qmth.scancloud.service.AnswerCardService;
import cn.com.qmth.scancloud.service.ExamService;
import cn.com.qmth.scancloud.service.SessionService;
import cn.com.qmth.scancloud.support.WithoutStackTrace;
import cn.com.qmth.scancloud.vo.ActiveTimeVo;
import cn.com.qmth.scancloud.vo.AnswerCardVo;
import cn.com.qmth.scancloud.vo.LogoutTimeVo;
import cn.com.qmth.scancloud.vo.apistatus.ApiStatusVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController
@Api(tags = "系统接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX)
@Aac(strict = BOOL.FALSE, auth = BOOL.TRUE)
public class SystemController extends BaseController {
	
	@Autowired
	private SessionService sessionService;
	
	@Autowired
	private SysProperty sysProperty;
	
	 @Autowired
	private AnswerCardService answerCardService;
	 
	 @Autowired
	 private ExamService examService;

	@WithoutStackTrace
	@ApiOperation(value = "心跳接口")
	@PostMapping("heartbeat")
	public ActiveTimeVo heartbeat() {
		User user=getAccessUser();
		ActiveTimeVo vo=new ActiveTimeVo();
		vo.setActiveTime(sessionService.updateUserSession(user));
		return vo;
	}

	@ApiOperation(value = "登出接口")
	@PostMapping("logout")
	public LogoutTimeVo logout() {
		User user=getAccessUser();
		sessionService.userLogout(user);
		LogoutTimeVo vo=new LogoutTimeVo();
		vo.setLogoutTime(System.currentTimeMillis());
		return vo;
	}
	
	@Aac(strict = BOOL.FALSE, auth = BOOL.FALSE)
	@ApiOperation(value = "服务端状态")
	@PostMapping("status")
	public ApiStatusVo status() {
		ApiStatusVo vo=new ApiStatusVo();
		vo.setFileUriPrefix(sysProperty.getFileUriPrefix());
		vo.setVersion(sysProperty.getVersion());
		vo.setTime(System.currentTimeMillis());
		return vo;
	}
	
    @ApiOperation(value = "答题卡卡格式查询")
    @PostMapping("/card/answer")
    public AnswerCardVo cardAnswer(@RequestParam Long examId,@RequestParam Integer number) {
        ExamEntity exam =examService.getById(examId);
        if(exam ==null ||!exam.getEnable()) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        AnswerCardEntity  card =answerCardService.findByExamAndNumber(examId, number);
        if(card ==null ) {
            throw new ParameterException("找不到卡格式");
        }
        return new AnswerCardVo(card);
    }
}