package cn.com.qmth.scancloud.controller.admin;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;

import cn.com.qmth.scancloud.bean.User;
import cn.com.qmth.scancloud.controller.BaseController;
import cn.com.qmth.scancloud.service.OmrTaskService;
import cn.com.qmth.scancloud.vo.task.TaskResultVo;
import cn.com.qmth.scancloud.vo.task.TaskSaveVo;
import cn.com.qmth.scancloud.vo.task.TaskStatusVo;
import cn.com.qmth.scancloud.vo.task.TaskVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.sf.json.JSONObject;

@RestController
@Api(tags = "识别对照接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/admin/check/omr/arbitrate")
@Aac(strict = BOOL.FALSE, auth = BOOL.TRUE)
public class ArbitrateController extends BaseController {

    protected static final Logger log = LoggerFactory.getLogger(ArbitrateController.class);

    @Autowired
    private OmrTaskService taskService;
    @Autowired
    private OmrTaskService omrTaskService;

    @ApiOperation(value = "识别对照仲裁任务获取")
    @RequestMapping(value = "/get", method = RequestMethod.POST)
    public TaskVo get(@RequestParam Long examId) {
        User user = getAccessUser();
        TaskVo task = taskService.getArbitrateTask(examId, user.getAccount());
        return task;
    }

    @ApiOperation(value = "识别对照仲裁结果提交")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    public TaskSaveVo save(@RequestBody TaskResultVo result) {
        User user = getAccessUser();
        return taskService.submitTask(result, user.getAccount());
    }

    @ApiOperation(value = "识别对照仲裁进度状态")
    @RequestMapping(value = "/status", method = RequestMethod.POST)
    public TaskStatusVo status(@RequestParam Long examId) {
        User user = getAccessUser();
        return taskService.getStatus(examId, true,user.getAccount());
    }
    
    @ApiOperation(value = "识别对照仲裁任务释放")
    @RequestMapping(value = "/release", method = RequestMethod.POST)
    public JSONObject release(@RequestParam Long examId) {
        User user = getAccessUser();
        omrTaskService.releaseByUser(examId, user.getAccount());
        JSONObject result = new JSONObject();
        result.accumulate("success", true);
        return result;
    }
}
