package cn.com.qmth.scancloud.controller.admin;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import javax.annotation.Resource;

import net.sf.json.JSONObject;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.task.AsyncTaskExecutor;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import cn.com.qmth.scancloud.controller.BaseController;
import cn.com.qmth.scancloud.entity.AnswerCardEntity;
import cn.com.qmth.scancloud.enums.LockType;
import cn.com.qmth.scancloud.service.AnswerCardService;
import cn.com.qmth.scancloud.task.CardSyncThread;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.boot.core.exception.ReentrantException;

@RestController
@Api(tags = "卡格式接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/admin/card")
@Aac(strict = BOOL.FALSE, auth = BOOL.TRUE)
public class CardController extends BaseController {

    protected static final Logger log = LoggerFactory.getLogger(CardController.class);

    @Autowired
    private AnswerCardService answerCardService;

    @Resource
    private ConcurrentService concurrentService;

    // @Qualifier("task-executor")
    @Autowired
    private AsyncTaskExecutor taskExecutor;

    @ApiOperation(value = "卡格式详情")
    @RequestMapping(value = "/info", method = RequestMethod.POST)
    public Map<String, Object> info(@RequestParam Long examId) {
        Map<String, Object> status = new HashMap<String, Object>();
        status.put("synching", concurrentService.isLocked(LockType.CARD_SYNC + "-" + examId));
        status.put("updateTime", System.currentTimeMillis());
        Map<String, Object> result = new HashMap<String, Object>();
        result.put("status", status);
        result.put("answer", answerCardService.listByExamId(examId));
        result.put("package", new ArrayList<>());
        return result;
    }

    @ApiOperation(value = "同步卡格式信息")
    @RequestMapping(value = "/sync", method = RequestMethod.POST)
    public JSONObject sync(@RequestParam Long examId) {
        if (!concurrentService.isLocked(LockType.CARD_SYNC + "-" + examId)) {
            taskExecutor.submit(new CardSyncThread(getAccessUser(), examId, answerCardService, concurrentService));
            JSONObject status = new JSONObject();
            status.accumulate("synching", true);
            status.accumulate("startTime", System.currentTimeMillis());
            return status;
        } else {
            throw new ReentrantException("正在同步卡格式，请稍后再试");
        }
    }

    @ApiOperation(value = "卡格式同步状态息")
    @RequestMapping(value = "/sync/status", method = RequestMethod.POST)
    public JSONObject syncStatus(@RequestParam Long examId) {
        JSONObject status = new JSONObject();
        status.accumulate("synching", concurrentService.isLocked(LockType.CARD_SYNC + "-" + examId));
        status.accumulate("updateTime", System.currentTimeMillis());
        return status;
    }

    @ApiOperation(value = "创建/修改题卡卡格式")
    @RequestMapping(value = "/answer/save", method = RequestMethod.POST)
    public JSONObject answerSave(@RequestParam Long examId, @RequestParam(required = false) Integer number,
            @RequestParam(required = false) String subjectCode, @RequestParam(required = false) String parameter,
            @RequestParam(required = false) String remark, @RequestParam Integer paperCount,
            @RequestParam Boolean singlePage, @RequestParam String md5, @RequestParam MultipartFile file) {
        if (concurrentService.getLock(LockType.CARD_SYNC + "-" + examId).tryLock()) {
            try {
                JSONObject result = new JSONObject();
                AnswerCardEntity card = answerCardService.save(getAccessUser(), examId, number, subjectCode, parameter,
                        remark, paperCount, singlePage, md5, file);
                result.accumulate("number", card.getNumber());
                result.accumulate("updateTime", System.currentTimeMillis());
                return result;
            } finally {
                concurrentService.getLock(LockType.CARD_SYNC + "-" + examId).unlock();
            }
        } else {
            throw new ReentrantException("正在同步卡格式，请稍后再试");
        }
    }

    @ApiOperation(value = "删除答题卡卡格式")
    @RequestMapping(value = "/answer/delete", method = RequestMethod.POST)
    public JSONObject answerDelete(@RequestParam Long examId, @RequestParam Integer number) {
        if (concurrentService.getLock(LockType.CARD_SYNC + "-" + examId).tryLock()) {
            try {
                answerCardService.delete(getAccessUser(), examId, number);
                JSONObject result = new JSONObject();
                result.accumulate("number", number);
                result.accumulate("updateTime", System.currentTimeMillis());
                return result;
            } finally {
                concurrentService.getLock(LockType.CARD_SYNC + "-" + examId).unlock();
            }
        } else {
            throw new ReentrantException("正在同步卡格式，请稍后再试");
        }
    }

    @ApiOperation(value = "创建/修改签到表卡格式")
    @RequestMapping(value = "/package/save", method = RequestMethod.POST)
    public JSONObject packageSave() {
        JSONObject result = new JSONObject();
        return result;
    }
}
