package cn.com.qmth.scancloud.controller.admin;

import java.io.IOException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletResponse;

import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.core.collection.PageResult;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.boot.tools.excel.ExcelWriter;
import com.qmth.boot.tools.excel.enums.ExcelType;
import com.qmth.boot.tools.excel.model.DataMap;

import cn.com.qmth.scancloud.bean.AbsentQueryDomain;
import cn.com.qmth.scancloud.controller.BaseController;
import cn.com.qmth.scancloud.entity.SubjectEntity;
import cn.com.qmth.scancloud.enums.ConditionType;
import cn.com.qmth.scancloud.enums.GroupType;
import cn.com.qmth.scancloud.service.StudentService;
import cn.com.qmth.scancloud.service.SubjectService;
import cn.com.qmth.scancloud.util.ResouceUtil;
import cn.com.qmth.scancloud.vo.AbsentInfoVo;
import cn.com.qmth.scancloud.vo.AbsentManualImportVo;
import cn.com.qmth.scancloud.vo.AbsentQueryVo;
import cn.com.qmth.scancloud.vo.OmrConditionsVo;
import cn.com.qmth.scancloud.vo.UpdateTimeVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController
@Api(tags = "缺考接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/admin/check")
@Aac(strict = BOOL.FALSE, auth = BOOL.TRUE)
public class CheckController extends BaseController {

	@Autowired
	private StudentService studentService;
	
	@Autowired
	private SubjectService subjectService;
	
	@ApiOperation(value = "所有可用的识别对照条件")
	@PostMapping("omr/conditions")
	public List<OmrConditionsVo> omrConditions() {
		List<OmrConditionsVo> list=new ArrayList<>();
		for(ConditionType c:ConditionType.values()) {
			OmrConditionsVo vo=new OmrConditionsVo();
			vo.setCode(c);
			vo.setName(c.getName());
			if (ConditionType.QUESTION_SINGLE_BLANK.equals(c)
            		||ConditionType.QUESTION_MULTI_BLANK.equals(c)
            		||ConditionType.SELECTIVE_EXCEED.equals(c)
            		||ConditionType.SELECTIVE_BLANK.equals(c)) {
				vo.setNeedValue(true);
			}else {
				vo.setNeedValue(false);
			}
			list.add(vo);
		}
		return list;
	}

	@ApiOperation(value = "缺考数据汇总")
	@PostMapping("absent/info")
	public AbsentInfoVo absentInfo(@RequestParam Long examId) {
		return studentService.absentInfo(examId);
	}

	@ApiOperation(value = "查询缺考数据")
	@RequestMapping(value = "absent/query", method = RequestMethod.POST)
	public PageResult<AbsentQueryVo> absentQuery(AbsentQueryDomain query) {
		return studentService.absentQuery(query);
	}

	@ApiOperation(value = "导出缺考数据")
	@RequestMapping(value = "absent/export", method = RequestMethod.POST)
	public void absentExport(AbsentQueryDomain query, HttpServletResponse response) throws IOException {
        String fileName = URLEncoder.encode("缺考数据", "UTF-8");
        response.setHeader("Content-Disposition", "inline; filename=" + fileName + ".xlsx");
        response.setContentType("application/vnd.ms-excel");

		List<AbsentQueryVo> list = studentService.absentExportList(query);
		List<DataMap> ret=new ArrayList<>();
		ExcelWriter writer = ExcelWriter.create(ExcelType.XLSX);
		
		String groupNameCol = null;
		if (GroupType.PACKAGE.equals(query.getGroupType())) {
			groupNameCol = "卷袋";
		}
		if (GroupType.SUBJECT.equals(query.getGroupType())) {
			groupNameCol = "科目代码";
		}
		if (GroupType.CAMPUS.equals(query.getGroupType())) {
			groupNameCol = "学习中心";
		}
		if (GroupType.EXAM_SITE.equals(query.getGroupType())) {
			groupNameCol = "考点";
		}
		if (GroupType.EXAM_ROOM.equals(query.getGroupType())) {
			groupNameCol = "考场";
		}
		if (GroupType.STUDENT_CODE.equals(query.getGroupType())) {
			groupNameCol = "学号";
		}
		if(CollectionUtils.isNotEmpty(list)) {
			Map<String,SubjectEntity> subMap=new HashMap<>();
			for(AbsentQueryVo vo:list) {
				DataMap map=new DataMap();
				ret.add(map);
				map.put(groupNameCol,vo.getGroupName());
				if (GroupType.SUBJECT.equals(query.getGroupType())) {
					map.put("科目名称",getSubjectName(subMap, vo.getGroupName(), query.getExamId()));
				}
				map.put("考生总数",getStringVal(vo.getTotalCount()));
				map.put("已扫描",getStringVal(vo.getScannedCount()));
				map.put("未扫描",getStringVal(vo.getUnexistCount()));
				map.put("指定缺考",getStringVal(vo.getManualAbsentCount()));
				map.put("识别缺考",getStringVal(vo.getOmrAbsentCount()));
				map.put("缺考异常",getStringVal(vo.getAbsentSuspectCount()));
			}
		}
		String[] head;
		if (GroupType.SUBJECT.equals(query.getGroupType())) {
			head=new String[] {groupNameCol,"科目名称","考生总数","已扫描","未扫描","指定缺考","识别缺考","缺考异常"};
		}else {
			head=new String[] {groupNameCol,"考生总数","已扫描","未扫描","指定缺考","识别缺考","缺考异常"};
		}
		writer.writeDataMaps("sheet1", null,head, ret.iterator());
		writer.output(response.getOutputStream());
	}
	
	private String getSubjectName(Map<String,SubjectEntity> subMap,String subjectCode,Long examId) {
		SubjectEntity  sub=subMap.get(subjectCode);
		if(subMap.get(subjectCode)==null) {
			sub=subjectService.findByExamIdAndCode(examId, subjectCode);
			if(sub==null) {
				throw new StatusException("未找到科目信息："+subjectCode);
			}
			subMap.put(subjectCode, sub);
		}
		return sub.getName();
	}
	
	private String getStringVal(Integer val) {
		if(val==null) {
			return "0";
		}
		return val.toString();
	}

	@PostMapping("absent/manual/import")
	@ApiOperation(value = "导入指定缺考名单")
	public AbsentManualImportVo absentManualImport(@RequestParam Long examId, @RequestParam MultipartFile file) {
		return studentService.absentManualImport(examId, file);
	}

	@ApiOperation(value = "下载导入指定缺考名单模板")
	@PostMapping("absent/manual/template")
	public void getImportTemplate() throws IOException {
		exportFile("指定缺考名单导入模板.xlsx", ResouceUtil.getStream("importtemplates/absentManualImportTemplate.xlsx"));
	}

	@ApiOperation(value = "更新为指定缺考")
	@PostMapping("absent/manual/update")
	public UpdateTimeVo absentManualUpdate(@RequestParam Long examId, @RequestParam String examNumber) {
		return studentService.absentManualUpdate(examId, examNumber);
	}

	@ApiOperation(value = "取消缺考标记嫌疑")
	@PostMapping("absent/suspect/remove")
	public UpdateTimeVo absentSuspectRemove(@RequestParam Long examId, @RequestParam String examNumber) {
		return studentService.absentSuspectRemove(examId, examNumber);
	}
}