package cn.com.qmth.scancloud.controller.admin;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;

import cn.com.qmth.scancloud.bean.User;
import cn.com.qmth.scancloud.controller.BaseController;
import cn.com.qmth.scancloud.service.BatchService;
import cn.com.qmth.scancloud.vo.UpdateTimeVo;
import cn.com.qmth.scancloud.vo.verify.VerifyTaskVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.sf.json.JSONObject;

@RestController
@Api(tags = "答题卡扫描实时审核接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/admin/verify/scan")
@Aac(strict = BOOL.FALSE, auth = BOOL.TRUE)
public class VerifyController extends BaseController {

    protected static final Logger log = LoggerFactory.getLogger(VerifyController.class);

    @Autowired
    private BatchService batchService;

    @ApiOperation(value = "获取答题卡扫描实时审核任务")
    @RequestMapping(value = "/answer/get", method = RequestMethod.POST)
    public VerifyTaskVo answerGet(@RequestParam Long examId) {
        User user = getAccessUser();
        return batchService.getVerifyTask(examId, user);
    }

    @ApiOperation(value = "答题卡扫描实时审核任务处理")
    @RequestMapping(value = "/answer/submit", method = RequestMethod.POST)
    public UpdateTimeVo answerSubmit(@RequestParam Long batchId, @RequestParam Boolean confirm) {
        User user = getAccessUser();
        batchService.verify(batchId, confirm,user);
        return result(System.currentTimeMillis());
    }
    @ApiOperation(value = "答题卡扫描实时审核任务释放")
    @RequestMapping(value = "/answer/release", method = RequestMethod.POST)
    public JSONObject answerRelease(@RequestParam Long examId) {
    	User user = getAccessUser();
    	batchService.releaseByUser(examId, user.getAccount());
        JSONObject result = new JSONObject();
        result.accumulate("success", true);
        return result;
    }
}
