package cn.com.qmth.scancloud.controller.scan;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;

import cn.com.qmth.scancloud.bean.BatchCreateDomain;
import cn.com.qmth.scancloud.bean.MismatchToggleDomain;
import cn.com.qmth.scancloud.bean.User;
import cn.com.qmth.scancloud.bean.answersave.AnswerDomain;
import cn.com.qmth.scancloud.controller.BaseController;
import cn.com.qmth.scancloud.service.AnswerCardService;
import cn.com.qmth.scancloud.service.BatchService;
import cn.com.qmth.scancloud.service.PaperService;
import cn.com.qmth.scancloud.vo.AnswerCardVo;
import cn.com.qmth.scancloud.vo.BatchFinishVo;
import cn.com.qmth.scancloud.vo.BatchVerifyVo;
import cn.com.qmth.scancloud.vo.MismatchToggleVo;
import cn.com.qmth.scancloud.vo.SheetUploadVo;
import cn.com.qmth.scancloud.vo.SliceUploadVo;
import cn.com.qmth.scancloud.vo.batch.AnswerSaveVo;
import cn.com.qmth.scancloud.vo.batch.BatchCreateVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController
@Api(tags = "扫描功能-答题卡接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/scan/answer")
@Aac(strict = BOOL.FALSE, auth = BOOL.TRUE)
public class AnswerController extends BaseController {

	@Autowired
	private AnswerCardService answerCardService;

	@Autowired
	private BatchService batchService;

	@Autowired
	private PaperService paperService;
	
    @ApiOperation(value = "答题卡扫描批次创建")
    @RequestMapping(value = "/batch/create", method = RequestMethod.POST)
    public BatchCreateVo batchCreate(@RequestBody BatchCreateDomain domain) {
        return batchService.batchCreate(domain,getAccessUser());
    }

	@ApiOperation(value = "答题卡卡格式列表")
	@RequestMapping(value = "/card/list", method = RequestMethod.POST)
	public List<AnswerCardVo> cardList(@RequestParam Long examId) {
		return answerCardService.listByExamIdForScanner(examId, getAccessUser());
	}

	@ApiOperation(value = "答题卡扫描结果保存")
	@RequestMapping(value = "/batch/save", method = RequestMethod.POST)
	public AnswerSaveVo batchSave(@RequestBody AnswerDomain domain) {
	    User user = getAccessUser();
		return batchService.batchSave(domain,user);
	}

	@ApiOperation(value = "答题卡扫描历史标记/取消异常")
	@RequestMapping(value = "/mismatch/toggle", method = RequestMethod.POST)
	public MismatchToggleVo mismatchToggle(MismatchToggleDomain domain) {
		return paperService.mismatchToggle(domain);
	}

	@ApiOperation(value = "答题卡扫描原图上传")
	@RequestMapping(value = "/sheet/upload", method = RequestMethod.POST)
	public SheetUploadVo sheetUpload(@RequestParam Long batchId, @RequestParam String examNumber,
			@RequestParam Integer paperNumber, @RequestParam Integer pageIndex, @RequestParam MultipartFile file,
			@RequestParam String md5) {
		return paperService.sheetUpload(batchId, examNumber, paperNumber, pageIndex, file, md5);
	}
	
	@ApiOperation(value = "答题卡扫描裁切图上传")
	@RequestMapping(value = "/slice/upload", method = RequestMethod.POST)
	public SliceUploadVo sliceUpload(@RequestParam Long batchId, @RequestParam String examNumber,
			@RequestParam Integer paperNumber, @RequestParam Integer pageIndex,@RequestParam Integer index, @RequestParam MultipartFile file,
			@RequestParam String md5) {
		return paperService.sliceUpload(batchId, examNumber, paperNumber, pageIndex,index, file, md5);
	}
	
	@ApiOperation(value = "答题卡扫描批次提交审核/查询审核结果")
	@RequestMapping(value = "/batch/verify", method = RequestMethod.POST)
	public BatchVerifyVo batchVerify(@RequestParam Long id) {
		return batchService.batchVerify(id);
	}
	
	@ApiOperation(value = "答题卡扫描批次完成")
	@RequestMapping(value = "/batch/finish", method = RequestMethod.POST)
	public BatchFinishVo batchFinish(@RequestParam Long id) {
		return batchService.batchFinish(id);
	}
}
