package cn.com.qmth.scancloud.multithread.consumer;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.boot.core.fss.store.FileStore;
import com.qmth.boot.core.retrofit.exception.RetrofitResponseError;
import com.qmth.boot.core.retrofit.utils.SignatureInfo;
import com.qmth.boot.core.retrofit.utils.UploadFile;
import com.qmth.boot.tools.io.IOUtils;
import com.qmth.boot.tools.signature.SignatureType;

import cn.com.qmth.scancloud.bean.SchoolSession;
import cn.com.qmth.scancloud.client.MarkingcloudApiClient;
import cn.com.qmth.scancloud.entity.ExamEntity;
import cn.com.qmth.scancloud.entity.PaperPageEntity;
import cn.com.qmth.scancloud.entity.StudentEntity;
import cn.com.qmth.scancloud.entity.StudentPaperEntity;
import cn.com.qmth.scancloud.enums.LockType;
import cn.com.qmth.scancloud.enums.UploadStatus;
import cn.com.qmth.scancloud.model.DataUploadDto;
import cn.com.qmth.scancloud.model.DataUploadResponse;
import cn.com.qmth.scancloud.model.FileUploadDto;
import cn.com.qmth.scancloud.model.FileUploadResponse;
import cn.com.qmth.scancloud.multithread.BatchConsumer;
import cn.com.qmth.scancloud.service.ExamService;
import cn.com.qmth.scancloud.service.FilePropertyService;
import cn.com.qmth.scancloud.service.PaperPageService;
import cn.com.qmth.scancloud.service.SessionService;
import cn.com.qmth.scancloud.service.StudentPaperService;
import cn.com.qmth.scancloud.service.StudentService;
import cn.com.qmth.scancloud.vo.StudentUploadVo;

@Scope("prototype")
@Service
public class UploadConsumer extends BatchConsumer<StudentUploadVo> {

	private static final Logger log = LoggerFactory.getLogger(UploadConsumer.class);

	@Autowired
	private StudentService studentService;

	@Autowired
	private ConcurrentService concurrentService;

	@Autowired
	private MarkingcloudApiClient markingcloudApiClient;

	@Autowired
	private PaperPageService paperPageService;

	@Autowired
	private StudentPaperService studentPaperService;

	@Autowired
	private FilePropertyService filePropertyService;

	@Autowired
	private FileStore fileStore;

	@Autowired
	private ExamService examService;
	
	@Autowired
	private SessionService sessionService;

	@Transactional
	@Override
	public void consume(StudentUploadVo s) {
		concurrentService.getReadWriteLock(LockType.STUDENT + "-" + s.getId()).writeLock().lock();
		try {
			SchoolSession ss=sessionService.getSchoolSession(s.getSchoolId());
			if(ss==null) {
				return;
			}
			SignatureInfo signatureInfo=new SignatureInfo(SignatureType.TOKEN,ss.getAccount() , ss.getMarkingCloudToken());
			StudentEntity se = studentService.getById(s.getId());
			try {
				if (UploadStatus.WAITING_UPLOAD.equals(se.getFileUploadStatus())) {
					uploadFile(s.getSchoolId(),signatureInfo,se);
				}
			} catch (Exception e) {
				se.setFileUploadStatus(UploadStatus.ERROR);
				log.error("上传文件出错.studentId:"+se.getId(), e);
			}
			try {
				if (UploadStatus.WAITING_UPLOAD.equals(se.getDataUploadStatus())) {
					uploadData(s.getSchoolId(),signatureInfo,se);
				}
			} catch (Exception e) {
				se.setDataUploadStatus(UploadStatus.ERROR);
				log.error("上传数据出错studentId:"+se.getId(), e);
			}
			studentService.saveOrUpdate(se);
		} catch (Exception e) {
			log.error("上传出错studentId:"+s.getId(), e);
		} finally {
			concurrentService.getReadWriteLock(LockType.STUDENT + "-" + s.getId()).writeLock().unlock();
		}
	}

	private void uploadFile(Long schoolId,SignatureInfo signatureInfo,StudentEntity student) {
		List<StudentPaperEntity> sps = studentPaperService.findByStudentId(student.getId());
		if (CollectionUtils.isEmpty(sps)) {
			student.setFileUploadStatus(UploadStatus.UPLOADED);
			return;
		}
		try {
			List<String> sheets = new ArrayList<>();
			List<String> slices = new ArrayList<>();
			for (StudentPaperEntity sp : sps) {
				List<PaperPageEntity> pages = paperPageService.listByPaperId(sp.getPaperId());
				if (CollectionUtils.isNotEmpty(sps)) {
					for (PaperPageEntity page : pages) {
						sheets.add(page.getSheetPath());
						slices.addAll(page.getSlicePath());
					}
				}
			}
			if (CollectionUtils.isNotEmpty(sheets)) {
				int index = 0;
				for (String s : sheets) {
					index++;
					FileUploadDto dto = new FileUploadDto();
					dto.setExamId(student.getExamId());
					dto.setExamNumber(student.getExamNumber());
					dto.setIndex(index);
					dto.setMd5(filePropertyService.getById(s).getMd5());
					FileUploadResponse res=null;
					try {
						res = markingcloudApiClient.sheetFileUpload(signatureInfo.toString(),dto.getExamId(), dto.getExamNumber(),
								dto.getIndex(), dto.getMd5(),
								UploadFile.build("file", getFileName(s), IOUtils.toByteArray(fileStore.read(s))));
						if (res==null||!res.getSuccess()) {
							throw new StatusException("文件上传失败,Success:false.studentId:"+student.getId());
						}
					} catch (RetrofitResponseError e) {
						if(e.getCode()==401) {
							sessionService.removeSchoolSession(schoolId);
							return;
						}else {
							throw e;
						}
					}
					
				}
			}
			if (CollectionUtils.isNotEmpty(slices)) {
				int index = 0;
				for (String s : slices) {
					index++;
					FileUploadDto dto = new FileUploadDto();
					dto.setExamId(student.getExamId());
					dto.setExamNumber(student.getExamNumber());
					dto.setIndex(index);
					dto.setMd5(filePropertyService.getById(s).getMd5());
					FileUploadResponse res=null;
					try {
						res = markingcloudApiClient.sliceFileUpload(signatureInfo.toString(),dto.getExamId(), dto.getExamNumber(),
								dto.getIndex(), dto.getMd5(),
								UploadFile.build("file", getFileName(s), IOUtils.toByteArray(fileStore.read(s))));
						if (res==null||!res.getSuccess()) {
							throw new StatusException("文件上传失败,Success:false.studentId:"+student.getId());
						}
					} catch (RetrofitResponseError e) {
						if(e.getCode()==401) {
							sessionService.removeSchoolSession(schoolId);
							return;
						}else {
							throw e;
						}
					}
				}
			}
		} catch (StatusException e) {
			throw e;
		} catch (Exception e) {
			throw new StatusException("文件处理出错studentId:"+student.getId(), e);
		}
		student.setFileUploadStatus(UploadStatus.UPLOADED);
	}

	private void uploadData(Long schoolId,SignatureInfo signatureInfo,StudentEntity student) {
		ExamEntity exam = examService.getById(student.getExamId());
		List<StudentPaperEntity> sps = studentPaperService.findByStudentId(student.getId());
		List<String> sheets = new ArrayList<>();
		List<String> slices = new ArrayList<>();
		List<PaperPageEntity> pageList = new ArrayList<>();
		if (CollectionUtils.isNotEmpty(sps)) {
			for (StudentPaperEntity sp : sps) {
				List<PaperPageEntity> pages = paperPageService.listByPaperId(sp.getPaperId());
				if (CollectionUtils.isNotEmpty(sps)) {
					pageList.addAll(pages);
					for (PaperPageEntity page : pages) {
						sheets.add(page.getSheetPath());
						slices.addAll(page.getSlicePath());
					}
				}
			}
		}
		if (CollectionUtils.isEmpty(sheets)) {
			throw new StatusException("数据上传出错,没有sheet数据.studentId:"+student.getId());
		}
		DataUploadDto dto = new DataUploadDto();
		dto.setSheetCount(sheets.size());
		dto.setSliceCount(slices.size());
		dto.setExamNumber(student.getExamNumber());
		dto.setManual(student.getAssigned());
		dto.setAbsent(student.getOmrAbsent());
		List<String> answers=new ArrayList<String>();
		int pageIndex = 0;
		for (PaperPageEntity p : pageList) {
			pageIndex++;
			if (pageIndex == 1) {
				dto.setPaperType(getPaperType(exam.getPaperTypeBarcodeContent(), p.getPaperType().getResult()));
				dto.setBatchCode(p.getBatchId() + "");
			}
			if(p.getQuestion()!=null&&CollectionUtils.isNotEmpty(p.getQuestion().getResult())) {
				answers.addAll(p.getQuestion().getResult());
			}
		}
		dto.setAnswers(StringUtils.join(answers, ","));
		List<DataUploadResponse> res=null;
		try {
			res = markingcloudApiClient.dataUpload(signatureInfo.toString(),exam.getId(),Arrays.asList(dto));
			if (CollectionUtils.isEmpty(res)||StringUtils.isBlank(res.get(0).getExamNumber())) {
				throw new StatusException("数据上传失败,Success:false.studentId:"+student.getId());
			}
		} catch (RetrofitResponseError e) {
			if(e.getCode()==401) {
				sessionService.removeSchoolSession(schoolId);
				return;
			}else {
				throw e;
			}
		}
		student.setDataUploadStatus(UploadStatus.UPLOADED);
	}

	private String getFileName(String path) {
		if (StringUtils.isBlank(path)) {
			throw new StatusException("文件路径有误:" + (path == null ? "" : path));
		}
		return path.substring(path.lastIndexOf("//") + 1);
	}

	private String getPaperType(List<String> list, String pt) {
		if (StringUtils.isBlank(pt)) {
			return "#";
		}

		if (CollectionUtils.isEmpty(list)) {
			if (pt.length() > 1) {
				return "#";
			}
			pt = pt.toUpperCase();
			int pAscii = (int) pt.charAt(0);
			if (pAscii >= 65 && pAscii <= 90) {
				return pt;
			} else {
				return "#";
			}
		} else {
			int index = list.indexOf(pt);
			if (index == -1) {
				return "#";
			}
			return String.valueOf((char) (index + 65));
		}
	}

}
