package cn.com.qmth.scancloud.service.impl;

import cn.com.qmth.scancloud.bean.User;
import cn.com.qmth.scancloud.client.MarkingcloudApiClient;
import cn.com.qmth.scancloud.client.MarkingcloudProperties;
import cn.com.qmth.scancloud.dao.AnswerCardDao;
import cn.com.qmth.scancloud.entity.*;
import cn.com.qmth.scancloud.enums.CardSource;
import cn.com.qmth.scancloud.enums.ExamMode;
import cn.com.qmth.scancloud.enums.SystemMode;
import cn.com.qmth.scancloud.model.AnswerCardInfo;
import cn.com.qmth.scancloud.service.*;
import cn.com.qmth.scancloud.vo.AnswerCardVo;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.github.jeffreyning.mybatisplus.service.MppServiceImpl;
import com.qmth.boot.core.exception.ParameterException;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.boot.core.fss.store.FileStore;
import com.qmth.boot.core.retrofit.exception.RetrofitResponseError;
import com.qmth.boot.core.retrofit.utils.SignatureInfo;
import com.qmth.boot.core.retrofit.utils.UploadFile;
import com.qmth.boot.core.security.exception.AuthorizationException;
import com.qmth.boot.tools.io.IOUtils;
import com.qmth.boot.tools.signature.SignatureType;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

@Service
public class AnswerCardServiceImpl extends MppServiceImpl<AnswerCardDao, AnswerCardEntity>
        implements AnswerCardService {

    @Autowired
    private PaperStructureService paperStructureService;

    @Autowired
    private ExamService examService;

    @Autowired
    private FileService fileService;

    @Autowired
    private FileStore fileStore;

    @Autowired
    private SubjectService subjectService;

    @Autowired
    private MarkingcloudApiClient apiClient;

    @Autowired
    private ScannerCardService scannerCardService;

    @Autowired
    private SessionService sessionService;

    @Autowired
    private MarkingcloudProperties properties;

    @Autowired
    private SystemConfigService systemConfigService;

    @Autowired
    private PaperService paperService;

    @Override
    public int getCountByExam(Long examId) {
        QueryWrapper<AnswerCardEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<AnswerCardEntity> lw = wrapper.lambda();
        lw.eq(AnswerCardEntity::getExamId, examId);
        return this.count(wrapper);
    }

    @Transactional
    @Override
    public AnswerCardEntity save(User user, Long examId, Integer number, String subjectCode, String parameter,
            String remark, Integer paperCount, Boolean singlePage, String md5, MultipartFile file) {
        ExamEntity exam = examService.getById(examId);
        if (ExamMode.K12.equals(exam.getMode()) && StringUtils.isBlank(subjectCode)) {
            throw new ParameterException("K12模式不允许创建通卡");
        }
        String subjectName = null;
        if (StringUtils.isNotBlank(subjectCode)) {
            SubjectEntity sub = subjectService.findByExamIdAndCode(examId, subjectCode);
            if (sub == null) {
                throw new ParameterException("未找到科目,请先同步或创建考生获取科目");
            }
            subjectName = sub.getName();
        }
        int studentCount = paperService.getCountByExamAndCardNumber(examId, number);
        if (studentCount != 0) {
            throw new ParameterException("卡格式已被使用，无法修改");
        }
        AnswerCardInfo answerCardInfo = null;
        byte[] bs;
        try {
            bs = IOUtils.toByteArray(file.getInputStream());
        } catch (IOException e) {
            throw new ParameterException("文件上传失败", e);
        }
        AnswerCardEntity entity = null;
        SystemConfigEntity conf = systemConfigService.findConfig();
        if (SystemMode.MARKING_CLOUD.equals(conf.getSystemMode())) {
            SignatureInfo signatureInfo = new SignatureInfo(SignatureType.TOKEN, user.getAccount(),
                    user.getMarkingCloudToken());
            answerCardInfo = apiClient
                    .saveAnswerCard(signatureInfo.toString(), examId, number, CardSource.CLIENT, subjectCode, parameter,
                            remark, paperCount, singlePage, false, md5, UploadFile.build("file", "", bs));
            number = answerCardInfo.getNumber();
            QueryWrapper<AnswerCardEntity> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(AnswerCardEntity::getExamId, examId);
            queryWrapper.lambda().eq(AnswerCardEntity::getNumber, number);
            entity = this.getOne(queryWrapper);
        } else {
            if (number != null) {
                QueryWrapper<AnswerCardEntity> queryWrapper = new QueryWrapper<>();
                queryWrapper.lambda().eq(AnswerCardEntity::getExamId, examId);
                queryWrapper.lambda().eq(AnswerCardEntity::getNumber, number);
                entity = this.getOne(queryWrapper);
            } else {
                number = findMaxCardNumberByExamId(examId) + 1;
            }
        }
        try {
            paperStructureService.updateByAnswerCard(file.getInputStream(), examId, number);
        } catch (IOException e) {
            throw new ParameterException("文件读取失败", e);
        }
        if (entity == null) {
            entity = new AnswerCardEntity();
            entity.setExamId(examId);
            entity.setNumber(number);
        }
        entity.setSubjectName(subjectName);
        entity.setPaperCount(paperCount);
        entity.setSinglePage(singlePage);
        entity.setParameter(parameter);
        entity.setRemark(remark);
        entity.setSubjectCode(subjectCode);
        entity.setSubjectName(subjectName);
        try {
            entity.setPath(fileService.uploadAnswerCard(file.getInputStream(), md5, examId, number));
        } catch (IOException e) {
            throw new ParameterException("文件上传失败", e);
        }
        entity.setMd5(md5);
        entity.setSource(CardSource.CLIENT);
        entity.setNeedAdapte(false);
        this.saveOrUpdateByMultiId(entity);
        return entity;
    }

    private Integer findMaxCardNumberByExamId(Long examId) {
        Integer number = 0;
        QueryWrapper<AnswerCardEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(AnswerCardEntity::getExamId, examId);
        List<AnswerCardEntity> list = this.list(queryWrapper);
        for (AnswerCardEntity card : list) {
            if (number < card.getNumber()) {
                number = card.getNumber();
            }
        }
        return number;
    }

    @Transactional
    @Override
    public void delete(User user, Long examId, Integer number) {
        if (examId == null) {
            throw new ParameterException("examId 不能为空");
        }
        if (number == null) {
            throw new ParameterException("number 不能为空");
        }
        int studentCount = paperService.getCountByExamAndCardNumber(examId, number);
        if (studentCount != 0) {
            throw new ParameterException("卡格式已被使用，无法删除");
        }
        UpdateWrapper<AnswerCardEntity> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(AnswerCardEntity::getExamId, examId);
        updateWrapper.lambda().eq(AnswerCardEntity::getNumber, number);
        int count = this.baseMapper.delete(updateWrapper);
        if (count == 0) {
            throw new ParameterException("未找到卡格式");
        }
        SystemConfigEntity conf = systemConfigService.findConfig();
        if (SystemMode.MARKING_CLOUD.equals(conf.getSystemMode())) {
            SignatureInfo signatureInfo = new SignatureInfo(SignatureType.TOKEN, user.getAccount(),
                    user.getMarkingCloudToken());
            apiClient.deleteAnswerCard(signatureInfo.toString(), examId, number);
        }
        paperStructureService.deleteByAnswerCard(examId, number);
        scannerCardService.deleteByExamAndCardNumber(examId, number);
    }

    @Override
    public List<AnswerCardVo> listByExamId(Long examId) {
        QueryWrapper<AnswerCardEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(AnswerCardEntity::getExamId, examId);
        List<AnswerCardEntity> list = this.list(queryWrapper);
        List<AnswerCardVo> result = new ArrayList<AnswerCardVo>();
        for (AnswerCardEntity answerCardEntity : list) {
            result.add(new AnswerCardVo(answerCardEntity));
        }
        return result;
    }

    @Override
    public List<AnswerCardVo> listByExamIdForScanner(Long examId, User scanner) {
        List<AnswerCardVo> list = listByExamId(examId);
        if (CollectionUtils.isNotEmpty(list)) {
            List<ScannerCardEntity> sc = scannerCardService.getListByExamAndDevice(examId, scanner.getAccount());
            if (CollectionUtils.isEmpty(sc)) {
                for (AnswerCardVo vo : list) {
                    vo.setEnable(true);
                }
            } else {
                Set<Integer> set = new HashSet<>();
                for (ScannerCardEntity e : sc) {
                    set.add(e.getCardNumber());
                }
                for (AnswerCardVo vo : list) {
                    if (set.contains(vo.getNumber())) {
                        vo.setEnable(true);
                    } else {
                        vo.setEnable(false);
                    }
                }
            }
        }
        return list;
    }

    @Transactional
    @Override
    public void sync(User user, Long examId) {
        SystemConfigEntity conf = systemConfigService.findConfig();
        if (SystemMode.STANDALONE.equals(conf.getSystemMode())) {
            return;
        }
        SignatureInfo signatureInfo = new SignatureInfo(SignatureType.TOKEN, user.getAccount(),
                user.getMarkingCloudToken());
        try {
            List<AnswerCardInfo> yun;
            yun = apiClient.getAnswerCardInfo(signatureInfo.toString(), examId, null);
            Set<Integer> numbers = new HashSet<Integer>();
            for (AnswerCardInfo answerCardInfo : yun) {
                numbers.add(answerCardInfo.getNumber());
                QueryWrapper<AnswerCardEntity> queryWrapper = new QueryWrapper<>();
                queryWrapper.lambda().eq(AnswerCardEntity::getExamId, examId);
                queryWrapper.lambda().eq(AnswerCardEntity::getNumber, answerCardInfo.getNumber());
                AnswerCardEntity entity = this.getOne(queryWrapper);
                if (entity == null) {
                    fileService.downLoadAnswerCard(examId, answerCardInfo.getNumber(), answerCardInfo.getMd5(),
                            properties.getFileServer() + answerCardInfo.getPath());
                    entity = new AnswerCardEntity();
                    entity.setExamId(examId);
                    entity.setNumber(answerCardInfo.getNumber());
                    entity.setPaperCount(answerCardInfo.getPaperCount());
                    entity.setSinglePage(answerCardInfo.getSinglePage());
                    entity.setParameter(answerCardInfo.getParameter());
                    entity.setRemark(answerCardInfo.getRemark());
                    entity.setSubjectCode(answerCardInfo.getSubjectCode());
                    entity.setSubjectName(answerCardInfo.getSubjectName());
                    entity.setPath(fileService.getAnswerCardUri(examId, answerCardInfo.getNumber()));
                    entity.setMd5(answerCardInfo.getMd5());
                    entity.setSource(CardSource.CLIENT);
                    entity.setNeedAdapte(false);
                    this.save(entity);
                    paperStructureService.updateByAnswerCard(fileStore.read(entity.getPath()), examId, entity.getNumber());
                } else if (!answerCardInfo.getMd5().equals(entity.getMd5())) {
                    int studentCount = paperService.getCountByExamAndCardNumber(examId, answerCardInfo.getNumber());
                    if (studentCount == 0) {
                        fileService.downLoadAnswerCard(examId, answerCardInfo.getNumber(), answerCardInfo.getMd5(),
                                properties.getFileServer() + answerCardInfo.getPath());
                        entity.setMd5(answerCardInfo.getMd5());
                        this.saveOrUpdateByMultiId(entity);
                        paperStructureService.updateByAnswerCard(fileStore.read(entity.getPath()), examId, entity.getNumber());
                    }
                }
            }
            QueryWrapper<AnswerCardEntity> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(AnswerCardEntity::getExamId, examId);
            List<AnswerCardEntity> list = this.list(queryWrapper);
            for (AnswerCardEntity answerCardEntity : list) {
                if (!numbers.contains(answerCardEntity.getNumber())) {
                    this.delete(user, examId, answerCardEntity.getNumber());
                }
            }
        } catch (RetrofitResponseError e) {
            if (e.getCode() == 401) {
                sessionService.userLogout(user);
                throw AuthorizationException.SIGNATURE_INVALID;
            } else {
                throw e;
            }
        } catch (Exception e) {
            throw new StatusException("answer card sync error: " + e.getMessage());
        }
    }

    @Override
    public AnswerCardEntity findByExamAndNumber(Long examId, Integer number) {
        if (examId == null) {
            throw new ParameterException("examId不能为空");
        }
        if (number == null) {
            throw new ParameterException("number不能为空");
        }
        QueryWrapper<AnswerCardEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<AnswerCardEntity> lw = wrapper.lambda();
        lw.eq(AnswerCardEntity::getExamId, examId);
        lw.eq(AnswerCardEntity::getNumber, number);
        return baseMapper.selectOne(wrapper);
    }

}
