package cn.com.qmth.scancloud.service.impl;

import cn.com.qmth.scancloud.bean.BatchCreateDomain;
import cn.com.qmth.scancloud.bean.BatchQueryDomain;
import cn.com.qmth.scancloud.bean.MismatchQueryDomain;
import cn.com.qmth.scancloud.bean.User;
import cn.com.qmth.scancloud.bean.answersave.AnswerDomain;
import cn.com.qmth.scancloud.bean.answersave.AnswerPage;
import cn.com.qmth.scancloud.bean.answersave.AnswerPaper;
import cn.com.qmth.scancloud.bean.answersave.StringResult;
import cn.com.qmth.scancloud.dao.BatchDao;
import cn.com.qmth.scancloud.entity.*;
import cn.com.qmth.scancloud.enums.*;
import cn.com.qmth.scancloud.exception.NotFoundExceptions;
import cn.com.qmth.scancloud.exception.ParameterExceptions;
import cn.com.qmth.scancloud.service.*;
import cn.com.qmth.scancloud.support.TaskLock;
import cn.com.qmth.scancloud.support.TaskLockUtil;
import cn.com.qmth.scancloud.util.BatchGetDataUtil;
import cn.com.qmth.scancloud.util.BatchSetDataUtil;
import cn.com.qmth.scancloud.util.PageUtil;
import cn.com.qmth.scancloud.vo.BatchFinishVo;
import cn.com.qmth.scancloud.vo.BatchQueryVo;
import cn.com.qmth.scancloud.vo.BatchVerifyVo;
import cn.com.qmth.scancloud.vo.batch.AnswerSaveVo;
import cn.com.qmth.scancloud.vo.batch.BatchCreateVo;
import cn.com.qmth.scancloud.vo.batch.Paper;
import cn.com.qmth.scancloud.vo.batch.Rescan;
import cn.com.qmth.scancloud.vo.batchdetail.BatchDetailVo;
import cn.com.qmth.scancloud.vo.batchdetail.BatchPageVo;
import cn.com.qmth.scancloud.vo.batchdetail.BatchPaperVo;
import cn.com.qmth.scancloud.vo.mismatchquery.MismatchPaperVo;
import cn.com.qmth.scancloud.vo.mismatchquery.MismatchQueryVo;
import cn.com.qmth.scancloud.vo.paper.PaperVo;
import cn.com.qmth.scancloud.vo.verify.VerifyPaperVo;
import cn.com.qmth.scancloud.vo.verify.VerifyStudentVo;
import cn.com.qmth.scancloud.vo.verify.VerifyTaskVo;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.core.collection.PageResult;
import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.boot.core.exception.ParameterException;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.boot.core.fss.store.FileStore;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class BatchServiceImpl extends ServiceImpl<BatchDao, BatchEntity> implements BatchService {

    @Autowired
    private ExamService examService;

    @Autowired
    private StudentService studentService;

    @Autowired
    private StudentPaperService studentPaperService;

    @Autowired
    private PaperService paperService;

    @Autowired
    private PaperPageService paperPageService;

    @Autowired
    private AnswerCardService answerCardService;

    @Autowired
    private ConcurrentService concurrentService;

    @Autowired
    private FileStore fileStore;

    @Autowired
    private OmrTaskService omrTaskService;

    @Autowired
    private OmrGroupService omrGroupService;

    @Override
    public void checkBatchStatus(BatchEntity b) {
        if (BatchStatus.FINISH.equals(b.getStatus())) {
            throw new ParameterException("批次已经完成");
        }
        if (VerifyStatus.WAITING.equals(b.getVerifyStatus())) {
            throw new ParameterException("批次正在审核中");
        }
        if (VerifyStatus.REJECT.equals(b.getVerifyStatus())) {
            throw new ParameterException("批次审核不通过");
        }
    }

    @Transactional
    @Override
    public BatchCreateVo batchCreate(BatchCreateDomain domain, User user) {
        if (domain.getExamId() == null) {
            throw new ParameterException("ExamId 不能为空");
        }
        if (CollectionUtils.isEmpty(domain.getExamNumbers())) {
            throw new ParameterException("准考证号不能为空");
        }
        ExamEntity exam = examService.getById(domain.getExamId());
        if (exam == null) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        BatchCreateVo vo = new BatchCreateVo();
        List<Rescan> rescan = new ArrayList<>();
        vo.setRescan(rescan);
        String packageCode = null;
        for (String examNumber : domain.getExamNumbers()) {
            StudentEntity student = studentService.findByExamAndExamNumber(domain.getExamId(), examNumber);
            if (student == null) {
                throw new ParameterException("未找到考生信息:" + examNumber);
            }
            if (exam.getScanByPackage() != null && exam.getScanByPackage()) {
                if (packageCode == null) {
                    packageCode = student.getPackageCode();
                } else {
                    if (!packageCode.equals(student.getPackageCode())) {
                        throw new ParameterException("考生[" + examNumber + "]卷袋编号与其它考生不一致");
                    }
                }
            }
            Rescan rc = getRescan(student);
            if (rc != null) {
                rescan.add(rc);
            }
        }
        if (exam.getScanByPackage() != null && exam.getScanByPackage()) {
            List<StudentEntity> ss = studentService.findByExamAndPackage(domain.getExamId(), packageCode);
            if (CollectionUtils.isEmpty(ss) || ss.size() != domain.getExamNumbers().size()) {
                throw new ParameterException("该批次考生数量与卷袋考生数量不一致");
            }
        }

        BatchEntity b = new BatchEntity();
        b.setExamId(domain.getExamId());
        b.setDevice(user.getAccount());
        b.setPackageCode(packageCode);
        b.setAssignedCount(0);
        b.setStatus(BatchStatus.INIT);
        b.setScanCount(0);
        this.save(b);
        vo.setId(b.getId());
        vo.setCreateTime(b.getCreateTime());
        return vo;
    }

    private Rescan getRescan(StudentEntity student) {
        Rescan res = new Rescan();
        res.setExamNumber(student.getExamNumber());
        List<PaperVo> papers = paperService.findStudentPaper(student.getId());
        List<Paper> resPapers = new ArrayList<>();
        res.setPapers(resPapers);
        if (CollectionUtils.isEmpty(papers)) {
            return null;
        }
        PaperVo first = papers.get(0);
        res.setBatchId(first.getBatchId());
        res.setDevice(first.getDevice());
        res.setCreateTime(first.getCreateTime());
        for (PaperVo paper : papers) {
            Paper p = new Paper();
            p.setId(paper.getId());
            p.setNumber(paper.getNumber());
            p.setMismatch(paper.getMismatch());
            List<String> pages = new ArrayList<>();
            p.setPages(pages);
            List<PaperPageEntity> paperPages = paperPageService.listByPaperId(paper.getId());
            if (CollectionUtils.isNotEmpty(paperPages)) {
                for (PaperPageEntity pp : paperPages) {
                    pages.add(pp.getSheetPath());
                }
            }
            resPapers.add(p);
        }
        return res;
    }

    @Override
    public int getCountByExam(Long examId) {
        QueryWrapper<BatchEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<BatchEntity> lw = wrapper.lambda();
        lw.eq(BatchEntity::getExamId, examId);
        return this.count(wrapper);
    }

    @Override
    public int getValidCountByExam(Long examId) {
        QueryWrapper<BatchEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<BatchEntity> lw = wrapper.lambda();
        lw.and(param -> {
            param.or().isNull(BatchEntity::getVerifyStatus);
            param.or().ne(BatchEntity::getVerifyStatus, VerifyStatus.REJECT);
        });
        lw.eq(BatchEntity::getExamId, examId);
        lw.gt(BatchEntity::getScanCount, 0);
        return this.count(wrapper);
    }

    @Override
    public int getVerifyCount(Long examId) {
        QueryWrapper<BatchEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<BatchEntity> lw = wrapper.lambda();
        lw.eq(BatchEntity::getExamId, examId);
        lw.eq(BatchEntity::getVerifyStatus, VerifyStatus.WAITING);
        return this.count(wrapper);
    }

    @Transactional
    @Override
    public AnswerSaveVo batchSave(AnswerDomain domain, User user) {
        Long batchId = domain.getBatchId();
        if (batchId == null) {
            throw new ParameterException("批次ID不能为空");
        }
        BatchEntity batch = this.getById(batchId);
        if (batch == null) {
            throw new ParameterException("批次信息未找到");
        }
        checkBatchStatus(batch);
        ExamEntity exam = examService.getById(batch.getExamId());
        if (exam == null) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        String examNumber = domain.getExamNumber();
        if (StringUtils.isBlank(examNumber)) {
            throw new ParameterException("准考证号不能为空");
        }
        StudentEntity student = studentService.findByExamAndExamNumber(batch.getExamId(), examNumber);
        if (student == null) {
            throw new ParameterException("考生信息未找到");
        }
        if (CollectionUtils.isEmpty(domain.getPapers())) {
            throw new ParameterException("保存张数不能为空");
        }
        concurrentService.getReadWriteLock(LockType.STUDENT + "-" + student.getId()).writeLock().lock();
        try {
            Integer cardNumber = domain.getCardNumber();
            if (cardNumber == null) {
                throw new ParameterException("卡格式编号不能为空");
            }
            AnswerCardEntity answerCard = answerCardService.findByExamAndNumber(batch.getExamId(), cardNumber);
            if (answerCard == null) {
                throw new ParameterException("卡格式信息未找到");
            }
            if (exam.getScanByPackage() != null && exam.getScanByPackage()) {
                if (!StringUtils.equals(batch.getPackageCode(), student.getPackageCode())) {
                    throw new ParameterException("与批次卷袋号不一致");
                }
            }
            if (StringUtils.isNotBlank(answerCard.getSubjectCode()) && !answerCard.getSubjectCode()
                    .equals(student.getSubjectCode())) {
                throw new ParameterException("卡格式科目代码不一致");
            }
            //不允许缺页时，保存张数需要与卡格式定义一致
            if (!exam.getAllowUnexistPaper() && domain.getPapers().size() != answerCard.getPaperCount()) {
                throw new ParameterException("卡格式张数不一致");
            }

            boolean studentAssigned = false;
            boolean studentQuestionFilled = false;
            for (AnswerPaper answerPaper : domain.getPapers()) {
                Integer number = answerPaper.getNumber();
                if (number == null) {
                    throw new ParameterException("Paper number不能为空");
                }
                if (number == 1) {
                    student.setCardNumber(cardNumber);
                }
                boolean paperAssigned = (answerPaper.getAssigned() == null ? false : answerPaper.getAssigned());
                boolean paperQuestionFilled = false;
                PaperEntity paper = paperService.findByStudentAndBatchAndNumber(student.getId(), batch.getId(), number);
                if (paper == null) {
                    paper = new PaperEntity();
                }
                paper.setExamId(batch.getExamId());
                paper.setStudentId(student.getId());
                paper.setBatchId(batchId);
                paper.setCardNumber(cardNumber);
                paper.setExamNumber(examNumber);
                paper.setNumber(number);
                paper.setPageCount(answerPaper.getPages().size());
                paper.setMismatch(false);
                paper.setQuestionFilled(false);
                paper.setAssigned(paperAssigned);
                if (paper.getId() == null) {
                    paperService.saveOrUpdate(paper);
                }
                if (paperAssigned) {
                    studentAssigned = true;
                }
                if (answerCard.getSinglePage() && answerPaper.getPages().size() != 1) {
                    throw new ParameterException("卡格式页数不一致");
                }
                if (!answerCard.getSinglePage() && answerPaper.getPages().size() != 2) {
                    throw new ParameterException("卡格式页数不一致");
                }
                for (AnswerPage answerPage : answerPaper.getPages()) {
                    Integer index = answerPage.getIndex();
                    if (index == null) {
                        throw new ParameterException("Page index不能为空");
                    }
                    if (StringUtils.isEmpty(answerPage.getSheetUri())) {
                        throw new ParameterException("Page SheetUri不能为空");
                    }
                    if (!fileStore.exist(answerPage.getSheetUri())) {
                        throw new StatusException("图片不存在:" + answerPage.getSheetUri());
                    }
                    if (CollectionUtils.isNotEmpty(answerPage.getSliceUri())) {
                        for (String spath : answerPage.getSliceUri()) {
                            if (!fileStore.exist(spath)) {
                                throw new ParameterException("图片不存在:" + spath);
                            }
                        }
                    }
                    if (number == 1 && index == 1) {
                        student.setPaperType(answerPage.getPaperType() == null || StringUtils
                                .isBlank(answerPage.getPaperType().getResult()) ?
                                "#" :
                                answerPage.getPaperType().getResult());
                        student.setOmrAbsent(
                                answerPage.getAbsent() == null ? false : answerPage.getAbsent().getResult());
                    }
                    PaperPageEntity page = paperPageService.findPaperIdAndIndex(paper.getId(), index);
                    if (page == null) {
                        page = new PaperPageEntity();
                    }
                    page.setExamId(batch.getExamId());
                    page.setStudentId(student.getId());
                    page.setPaperId(paper.getId());
                    page.setBatchId(batchId);
                    page.setSheetPath(answerPage.getSheetUri());
                    page.setSlicePath(answerPage.getSliceUri());
                    page.setPageIndex(index);
                    page.setAbsent(answerPage.getAbsent());
                    page.setBreach(answerPage.getBreach());
                    checkPaperTypeValid(exam.getPaperTypeBarcodeContent(), answerPage.getPaperType());
                    page.setPaperType(answerPage.getPaperType());
                    page.setQuestion(answerPage.getQuestion());
                    page.setSelective(answerPage.getSelective());
                    page.setRecogData(answerPage.getRecogData());
                    if (answerPage.getQuestion() != null && checkQuestionFilled(answerPage.getQuestion().getResult())) {
                        paperQuestionFilled = true;
                        studentQuestionFilled = true;
                    }
                    paperPageService.saveOrUpdateByMultiId(page);
                }
                paper.setQuestionFilled(paperQuestionFilled);
                paperService.saveOrUpdate(paper);
                if (exam.getEnableSyncVerify() == null || !exam.getEnableSyncVerify() || !studentAssigned) {
                    omrTaskService.deleteByStudentId(student.getExamId(), user, student.getId());
                    studentPaperService.update(student.getId(), paper.getNumber(), paper.getId());
                }
            }
            updateAssignedCount(batchId);
            updateScanCount(batchId);
            if (exam.getEnableSyncVerify() == null || !exam.getEnableSyncVerify() || !studentAssigned) {
                student.setStatus(ScanStatus.SCANNED);
                student.setIncomplete(domain.getPapers().size() != answerCard.getPaperCount());
                student.setAssigned(studentAssigned);
                student.setQuestionFilled(studentQuestionFilled);
                student.setFileUploadStatus(UploadStatus.WAITING_UPLOAD);
                student.setDataUploadStatus(UploadStatus.WAITING_UPLOAD);
                studentService.updateById(student);
                List<OmrGroupEntity> gs = omrGroupService.findByExamIdAndFixed(exam.getId(), true);
                if (CollectionUtils.isNotEmpty(gs)) {
                    for (OmrGroupEntity g : gs) {
                        omrTaskService.saveTask(g, student.getId());
                    }
                }
            }
            AnswerSaveVo vo = new AnswerSaveVo();
            vo.setScanCount(paperService.findCountByBatch(batchId));
            vo.setUpdateTime(new Date().getTime());
            return vo;
        } finally {
            concurrentService.getReadWriteLock(LockType.STUDENT + "-" + student.getId()).writeLock().unlock();
        }
    }

    private void updateBatchAssignedStudentData(Long batchId, User user) {
        List<PaperEntity> papers = paperService.findAssigneByBatch(batchId);
        if (CollectionUtils.isEmpty(papers)) {
            return;
        }
        Map<Long, StudentEntity> students = new HashMap<>();
        try {
            for (PaperEntity paper : papers) {
                StudentEntity student = students.get(paper.getStudentId());
                if (student == null) {
                    student = studentService.getById(paper.getStudentId());
                    students.put(student.getId(), student);
                    concurrentService.getReadWriteLock(LockType.STUDENT + "-" + student.getId()).writeLock().lock();
                }
                if (paper.getNumber() == 1) {
                    student.setCardNumber(paper.getCardNumber());
                    PaperPageEntity page = paperPageService.findPaperIdAndIndex(paper.getId(), 1);
                    student.setPaperType(
                            page.getPaperType() == null || StringUtils.isBlank(page.getPaperType().getResult()) ?
                                    "#" :
                                    page.getPaperType().getResult());
                    student.setOmrAbsent(page.getAbsent() == null ? false : page.getAbsent().getResult());
                }
                student.setStatus(ScanStatus.SCANNED);
                student.setAssigned(true);
                //默认实时审核都不允许缺页
                student.setIncomplete(false);
                if (paper.getQuestionFilled() != null && paper.getQuestionFilled()) {
                    student.setQuestionFilled(true);
                }
                student.setFileUploadStatus(UploadStatus.WAITING_UPLOAD);
                student.setDataUploadStatus(UploadStatus.WAITING_UPLOAD);
                omrTaskService.deleteByStudentId(student.getExamId(), user, student.getId());
                studentPaperService.update(student.getId(), paper.getNumber(), paper.getId());
            }
            for (StudentEntity student : students.values()) {
                studentService.updateById(student);
                List<OmrGroupEntity> gs = omrGroupService.findByExamIdAndFixed(student.getExamId(), true);
                if (CollectionUtils.isNotEmpty(gs)) {
                    for (OmrGroupEntity g : gs) {
                        omrTaskService.saveTask(g, student.getId());
                    }
                }
            }
        } finally {
            for (StudentEntity student : students.values()) {
                concurrentService.getReadWriteLock(LockType.STUDENT + "-" + student.getId()).writeLock().unlock();
            }
        }
    }

    private boolean checkPaperTypeValid(List<String> list, StringResult r) {
        if (r == null) {
            return true;
        }
        if (!OmrType.BARCODE.equals(r.getType())) {
            return true;
        }
        if (CollectionUtils.isEmpty(list)) {
            return true;
        }
        if ("#".equals(r.getResult())) {
            return true;
        }
        if (!list.contains(r.getResult())) {
            r.setResult("?");
            return false;
        }
        return true;
    }

    private boolean checkQuestionFilled(List<String> list) {
        if (CollectionUtils.isEmpty(list)) {
            return false;
        }
        for (String s : list) {
            if (!"#".equals(s)) {
                return true;
            }
        }
        return false;
    }

    @Transactional
    @Override
    public void verify(Long batchId, Boolean comfirm, User user) {
        BatchEntity entity = this.getById(batchId);
        if (entity == null) {
            throw new ParameterException("批次不存在");
        }
        if (!VerifyStatus.WAITING.equals(entity.getVerifyStatus())) {
            throw new ParameterException("批次已被处理");
        }
        if (comfirm) {
            entity.setVerifyStatus(VerifyStatus.CONFIRM);
            updateBatchAssignedStudentData(batchId, user);
        } else {
            entity.setVerifyStatus(VerifyStatus.REJECT);
        }
        entity.setUpdateTime(System.currentTimeMillis());
        this.saveOrUpdate(entity);
    }

    @Transactional
    @Override
    public BatchVerifyVo batchVerify(Long id) {
        BatchEntity entity = this.getById(id);
        if (entity == null) {
            throw new ParameterException("批次不存在");
        }
        Integer count = paperService.findCountByBatch(id);
        if (count == null || count == 0) {
            throw new ParameterException("该批次未上传图片");
        }
        if (entity.getVerifyStatus() == null) {
            ExamEntity exam = examService.getById(entity.getExamId());
            if (exam.getEnableSyncVerify() == null || !exam.getEnableSyncVerify()) {
                throw new ParameterException("考试未启用实时审核");
            }
            entity.setVerifyStatus(VerifyStatus.WAITING);
            this.saveOrUpdate(entity);
        }
        BatchVerifyVo vo = new BatchVerifyVo();
        vo.setStatus(entity.getVerifyStatus());
        vo.setUpdateTime(entity.getUpdateTime());
        return vo;
    }

    @Transactional
    @Override
    public BatchFinishVo batchFinish(Long id) {
        BatchEntity entity = this.getById(id);
        if (entity == null) {
            throw new ParameterException("批次不存在");
        }
        checkBatchStatus(entity);
        ExamEntity exam = examService.getById(entity.getExamId());
        if (exam.getScanByPackage() != null && exam.getScanByPackage()) {
            checkStudentCount(entity);
        }
        entity.setStatus(BatchStatus.FINISH);
        this.saveOrUpdate(entity);
        BatchFinishVo vo = new BatchFinishVo();
        vo.setStatus(entity.getStatus());
        vo.setUpdateTime(System.currentTimeMillis());
        return vo;
    }

    private void checkStudentCount(BatchEntity entity) {
        List<PaperEntity> papers = paperService.findByBatch(entity.getId());
        List<StudentEntity> studetns = studentService.findByExamAndPackage(entity.getExamId(), entity.getPackageCode());
        if (CollectionUtils.isEmpty(papers) && CollectionUtils.isEmpty(studetns)) {
            return;
        }
        if (CollectionUtils.isEmpty(papers) || CollectionUtils.isEmpty(studetns)) {
            throw new ParameterException("卷袋考生有未扫描");
        }
        Set<Long> ps = new HashSet<>();
        Set<Long> ss = new HashSet<>();
        for (PaperEntity p : papers) {
            ps.add(p.getStudentId());
        }
        for (StudentEntity s : studetns) {
            ss.add(s.getId());
        }
        if (ps.size() != ss.size()) {
            throw new ParameterException("卷袋考生有未扫描");
        }
        for (Long p : ps) {
            if (!ss.contains(p)) {
                throw new ParameterException("卷袋考生有未扫描");
            }
        }
    }

    @Override
    public List<BatchEntity> listByExam(Long examId) {
        QueryWrapper<BatchEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<BatchEntity> lw = wrapper.lambda();
        lw.eq(BatchEntity::getExamId, examId);
        return this.list(wrapper);
    }

    @Transactional
    @Override
    public void updateScanCount(Long id) {
        baseMapper.updateScanCount(id);
    }

    @Transactional
    @Override
    public void updateAssignedCount(Long id) {
        baseMapper.updateAssignedCount(id);
    }

    @Override
    public PageResult<BatchQueryVo> batchQuery(BatchQueryDomain query) {
        if (query.getExamId() == null) {
            throw new ParameterException("examId 不能为空");
        }
        IPage<BatchQueryVo> iPage = baseMapper
                .batchQueryPage(new Page<BatchQueryVo>(query.getPageNumber(), query.getPageSize()), query);
        List<BatchQueryVo> list = iPage.getRecords();
        if (CollectionUtils.isNotEmpty(list)) {
            new BatchSetDataUtil<BatchQueryVo>() {

                @Override
                protected void setData(List<BatchQueryVo> dataList) {
                    List<Long> batchIds = dataList.stream().map(dto -> dto.getId()).distinct()
                            .collect(Collectors.toList());
                    List<BatchQueryVo> ret = baseMapper.findAssignedCount(batchIds);
                    if (ret != null && ret.size() > 0) {
                        Map<Long, Integer> countMap = new HashMap<>();
                        for (BatchQueryVo item : ret) {
                            countMap.put(item.getId(), item.getAssignedCount());
                        }
                        for (BatchQueryVo vo : dataList) {
                            Integer c = countMap.get(vo.getId());
                            vo.setAssignedCount(c == null ? 0 : c);
                        }
                    }
                }
            }.setDataForBatch(list, 20);
        }
        return PageUtil.of(iPage);
    }

    @Override
    public List<BatchDetailVo> batchDetail(Long id) {
        if (getById(id) == null) {
            throw new ParameterException("没有找到批次信息");
        }
        List<BatchDetailVo> list = baseMapper.batchDetailList(id);
        if (CollectionUtils.isEmpty(list)) {
            throw new ParameterException("没有扫描数据");
        }
        Map<Long, BatchDetailVo> map = new HashMap<>();
        Map<Long, BatchPaperVo> paperMap = new HashMap<>();
        for (BatchDetailVo vo : list) {
            map.put(vo.getId(), vo);
        }
        List<PaperEntity> paperList = paperService.findByBatch(id);
        for (PaperEntity p : paperList) {
            BatchDetailVo vo = map.get(p.getStudentId());
            if (vo == null) {
                continue;
            }
            List<BatchPaperVo> papers = vo.getPapers();
            if (papers == null) {
                papers = new ArrayList<>();
                vo.setPapers(papers);
            }
            BatchPaperVo pvo = new BatchPaperVo();
            pvo.setNumber(p.getNumber());
            pvo.setAssigned(p.getAssigned());
            papers.add(pvo);
            paperMap.put(p.getId(), pvo);
        }
        List<PaperPageEntity> paperPageList = paperPageService.findByBatch(id);
        for (PaperPageEntity p : paperPageList) {
            BatchPaperVo pvo = paperMap.get(p.getPaperId());
            if (pvo == null) {
                continue;
            }
            List<BatchPageVo> pages = pvo.getPages();
            if (pages == null) {
                pages = new ArrayList<>();
                pvo.setPages(pages);
            }
            BatchPageVo pagevo = new BatchPageVo();
            pagevo.setIndex(p.getPageIndex());
            pagevo.setSheetUri(p.getSheetPath());
            pagevo.setSliceUri(p.getSlicePath());
            pagevo.setAbsent(
                    p.getAbsent() == null || p.getAbsent().getResult() == null ? false : p.getAbsent().getResult());
            pages.add(pagevo);
        }
        for (BatchDetailVo vo : list) {
            disposeStudent(vo);
        }
        return list;
    }

    private void disposeStudent(BatchDetailVo vo) {
        if (CollectionUtils.isEmpty(vo.getPapers())) {
            vo.setAssigned(false);
            vo.setOmrAbsent(false);
            return;
        }
        boolean assigned = false;
        boolean omrAbsent = false;
        for (BatchPaperVo p : vo.getPapers()) {
            if (p.getAssigned() != null && p.getAssigned()) {
                assigned = true;
            }
            if (CollectionUtils.isEmpty(p.getPages())) {
                for (BatchPageVo page : p.getPages()) {
                    if (page.getAbsent() != null && page.getAbsent()) {
                        omrAbsent = true;
                    }
                }
            }
        }
        vo.setAssigned(assigned);
        vo.setOmrAbsent(omrAbsent);
    }

    @Override
    public PageResult<MismatchQueryVo> mismatchQuery(MismatchQueryDomain query) {
        if (query.getExamId() == null) {
            throw new ParameterException("examId 不能为空");
        }
        IPage<MismatchQueryVo> iPage = baseMapper
                .mismatchQueryPage(new Page<MismatchQueryVo>(query.getPageNumber(), query.getPageSize()), query);
        if (CollectionUtils.isNotEmpty(iPage.getRecords())) {
            Map<String, MismatchQueryVo> map = new HashMap<>();

            for (MismatchQueryVo vo : iPage.getRecords()) {
                vo.setPapers(new ArrayList<>());
                map.put(vo.getBatchId() + "-" + vo.getStudentId(), vo);
            }
            List<PaperEntity> paperList = new BatchGetDataUtil<PaperEntity, MismatchQueryVo>() {

                @Override
                public List<PaperEntity> getData(List<MismatchQueryVo> paramList) {
                    return paperService.listByMismatchQuery(paramList);
                }
            }.getDataForBatch(iPage.getRecords(), 20);

            if (CollectionUtils.isNotEmpty(paperList)) {
                Map<Long, MismatchPaperVo> paperMap = new HashMap<>();
                for (PaperEntity p : paperList) {
                    MismatchQueryVo vo = map.get(p.getBatchId() + "-" + p.getStudentId());
                    if (vo == null) {
                        continue;
                    }
                    List<MismatchPaperVo> papers = vo.getPapers();
                    if (papers == null) {
                        papers = new ArrayList<>();
                        vo.setPapers(papers);
                    }
                    MismatchPaperVo pvo = new MismatchPaperVo();
                    pvo.setId(p.getId());
                    pvo.setNumber(p.getNumber());
                    papers.add(pvo);
                    paperMap.put(p.getId(), pvo);
                }

                List<Long> paperIds = paperList.stream().map(p -> p.getId()).collect(Collectors.toList());
                List<PaperPageEntity> paperPageList = new BatchGetDataUtil<PaperPageEntity, Long>() {

                    @Override
                    public List<PaperPageEntity> getData(List<Long> paramList) {
                        return paperPageService.listByPaperList(paramList);
                    }
                }.getDataForBatch(paperIds, 20);

                if (CollectionUtils.isNotEmpty(paperPageList)) {
                    for (PaperPageEntity p : paperPageList) {
                        MismatchPaperVo pvo = paperMap.get(p.getPaperId());
                        if (pvo == null) {
                            continue;
                        }
                        List<String> pages = pvo.getPages();
                        if (pages == null) {
                            pages = new ArrayList<>();
                            pvo.setPages(pages);
                        }
                        pages.add(p.getSheetPath());
                    }
                }
            }
        }
        return PageUtil.of(iPage);
    }

    @Override
    public VerifyTaskVo getVerifyTask(Long examId, User user) {
        int retry = 1;
        VerifyTaskVo task = null;
        while (task == null) {
            List<BatchEntity> list = this.findUnVerify(examId, retry, 20, VerifyStatus.WAITING).getRecords();
            if (list.isEmpty()) {
                break;
            }
            for (BatchEntity t : list) {
                if (this.apply(t, user.getAccount())) {
                    task = toTaskVo(t);
                    break;
                }
            }
            if (task == null) {
                retry++;
            }
        }
        if (task == null) {
            throw NotFoundExceptions.NO_SYNC_VERIFY_TASK;
        }
        return task;
    }

    private VerifyTaskVo toTaskVo(BatchEntity t) {
        VerifyTaskVo vo = new VerifyTaskVo();
        vo.setBatchId(t.getId());
        vo.setCreateTime(t.getCreateTime());
        vo.setDevice(t.getDevice());
        vo.setPackageCode(t.getPackageCode());

        List<PaperPageEntity> pageList = paperPageService.findByBatch(t.getId());
        Map<Long, List<String>> pageMap = new HashMap<>();
        for (PaperPageEntity paperPageEntity : pageList) {
            List<String> url = pageMap.get(paperPageEntity.getPaperId());
            if (url == null) {
                url = new ArrayList<>();
            }
            url.add(paperPageEntity.getSheetPath());
            pageMap.put(paperPageEntity.getPaperId(), url);
        }

        List<PaperEntity> paperList = paperService.findByBatch(t.getId());
        Map<String, List<VerifyPaperVo>> paperMap = new HashMap<String, List<VerifyPaperVo>>();
        for (PaperEntity paper : paperList) {
            List<VerifyPaperVo> verifyPaperVos = paperMap.get(paper.getExamNumber());
            if (verifyPaperVos == null) {
                verifyPaperVos = new ArrayList<VerifyPaperVo>();
            }
            VerifyPaperVo paperVo = new VerifyPaperVo();
            paperVo.setAssgined(paper.getAssigned());
            paperVo.setNumber(paper.getNumber());
            paperVo.setPages(pageMap.get(paper.getId()));
            verifyPaperVos.add(paperVo);
            paperMap.put(paper.getExamNumber(), verifyPaperVos);
        }

        List<VerifyStudentVo> students = new ArrayList<VerifyStudentVo>();
        List<BatchDetailVo> list = baseMapper.batchDetailList(t.getId());
        for (BatchDetailVo detailVo : list) {
            VerifyStudentVo student = new VerifyStudentVo();
            student.setExamNumber(detailVo.getExamNumber());
            student.setName(detailVo.getName());
            student.setStudentCode(detailVo.getStudentCode());
            student.setSubjectCode(detailVo.getSubjectCode());
            student.setSubjectName(detailVo.getSubjectName());
            student.setPapers(paperMap.get(detailVo.getExamNumber()));
            students.add(student);
        }
        vo.setStudents(students);
        return vo;
    }

    @Override
    public boolean apply(BatchEntity t, String user) {
        TaskLock taskLock = TaskLockUtil.getVerifyTask(t.getExamId().toString());
        boolean lock = taskLock.add(t.getId(), user);
        // 上锁失败直接返回
        if (!lock) {
            return false;
        }
        // 重复校验任务状态
        if (VerifyStatus.WAITING.equals(t.getVerifyStatus())) {
            return true;
        } else {
            taskLock.remove(t.getId().toString());
            return false;
        }
    }

    @Override
    public void releaseByUser(Long examId, String account) {
        TaskLock taskLock = TaskLockUtil.getVerifyTask(examId.toString());
        taskLock.clear(account);
    }

    @Override
    public void releaseByTask(BatchEntity t) {
        TaskLock taskLock = TaskLockUtil.getVerifyTask(t.getExamId().toString());
        taskLock.remove(t.getId().toString());
    }

    @Override
    public boolean hasApplied(BatchEntity t, String user) {
        TaskLock taskLock = TaskLockUtil.getVerifyTask(t.getExamId().toString());
        return taskLock.exist(t.getId(), user);
    }

    @Override
    public IPage<BatchEntity> findUnVerify(Long examId, int pageNumber, int pageSize, VerifyStatus status) {
        return this.baseMapper.findUnVerify(new Page<BatchEntity>(pageNumber, pageSize), examId, status);
    }

    @Override
    public List<String> batchScanner(Long examId) {
        return baseMapper.batchScanner(examId);
    }

}
