package cn.com.qmth.scancloud.service.impl;

import cn.com.qmth.scancloud.bean.*;
import cn.com.qmth.scancloud.dao.PaperDao;
import cn.com.qmth.scancloud.entity.*;
import cn.com.qmth.scancloud.enums.LockType;
import cn.com.qmth.scancloud.enums.ScanStatus;
import cn.com.qmth.scancloud.enums.UploadStatus;
import cn.com.qmth.scancloud.service.*;
import cn.com.qmth.scancloud.vo.*;
import cn.com.qmth.scancloud.vo.answerquery.StudentPaperVo;
import cn.com.qmth.scancloud.vo.mismatchquery.MismatchQueryVo;
import cn.com.qmth.scancloud.vo.paper.PaperVo;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.boot.core.exception.ParameterException;
import com.qmth.boot.core.exception.StatusException;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.Date;
import java.util.List;

@Service
public class PaperServiceImpl extends ServiceImpl<PaperDao, PaperEntity> implements PaperService {

    protected static Logger log = LoggerFactory.getLogger(PaperServiceImpl.class);

    @Autowired
    private StudentPaperService studentPaperService;

    @Autowired
    private BatchService batchService;

    @Autowired
    private StudentService studentService;

    @Autowired
    private ConcurrentService concurrentService;

    @Autowired
    private FileService fileService;

    @Autowired
    private FilePropertyService filePropertyService;

    @Autowired
    private PaperPageService paperPageService;

    @Autowired
    private OmrTaskService omrTaskService;

    @Override
    public List<PaperEntity> findStudentPaperEffective(Long studentId) {
        return baseMapper.findStudentPaperEffective(studentId);
    }

    @Override
    public List<PaperVo> findStudentPaper(Long studentId) {
        return baseMapper.findStudentPaper(studentId);
    }

    @Override
    public int getMismatchCountByExam(Long examId) {
        QueryWrapper<PaperEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<PaperEntity> lw = wrapper.lambda();
        lw.eq(PaperEntity::getExamId, examId);
        lw.eq(PaperEntity::getMismatch, true);
        return this.count(wrapper);
    }

    @Override
    public PaperEntity findByStudentAndBatchAndNumber(Long studentId, Long batchId, Integer number) {
        QueryWrapper<PaperEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<PaperEntity> lw = wrapper.lambda();
        lw.eq(PaperEntity::getBatchId, batchId);
        lw.eq(PaperEntity::getStudentId, studentId);
        lw.eq(PaperEntity::getNumber, number);
        return baseMapper.selectOne(wrapper);
    }

    @Transactional
    @Override
    public void deleteByBatchIdAndStudentId(Long batchId, Long studentId) {
        if (batchId == null) {
            throw new ParameterException("批次ID不能为空");
        }
        if (studentId == null) {
            throw new ParameterException("考生ID不能为空");
        }
        QueryWrapper<PaperEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<PaperEntity> lw = wrapper.lambda();
        lw.eq(PaperEntity::getBatchId, batchId);
        lw.eq(PaperEntity::getStudentId, studentId);
        this.baseMapper.delete(wrapper);
    }

    @Transactional
    @Override
    public MismatchToggleVo mismatchToggle(MismatchToggleDomain domain) {
        if (domain.getPaperId() == null) {
            throw new ParameterException("PaperId不能为空");
        }
        if (domain.getEnable() == null) {
            throw new ParameterException("Enable不能为空");
        }
        PaperEntity paper = this.getById(domain.getPaperId());
        if (paper == null) {
            throw new ParameterException("未找到paper信息");
        }
        paper.setMismatch(domain.getEnable());
        this.updateById(paper);

        MismatchToggleVo vo = new MismatchToggleVo();
        vo.setMismatch(domain.getEnable());
        vo.setPaperId(domain.getPaperId());
        vo.setUpdateTime(new Date().getTime());
        return vo;
    }

    @Transactional
    @Override
    public SheetUploadVo sheetUpload(Long batchId, String examNumber, Integer paperNumber, Integer pageIndex,
            MultipartFile multipartFile, String md5) {
        BatchEntity batch = batchService.getById(batchId);
        if (batch == null) {
            throw new ParameterException("批次信息未找到");
        }
        batchService.checkBatchStatus(batch);
        StudentEntity student = studentService.findByExamAndExamNumber(batch.getExamId(), examNumber);
        if (student == null) {
            throw new ParameterException("考生信息未找到");
        }
        concurrentService.getReadWriteLock(LockType.STUDENT + "-" + student.getId()).writeLock().lock();
        try {
            String path = fileService
                    .uploadSheet(multipartFile.getInputStream(), md5, batch.getExamId(), batchId, examNumber,
                            paperNumber, pageIndex);
            filePropertyService.save(batch.getExamId(), path, md5);
            SheetUploadVo vo = new SheetUploadVo();
            vo.setUri(path);
            vo.setUpdateTime(new Date().getTime());
            return vo;
        } catch (Exception e) {
            log.error("原图上传失败，batchId=" + batch + ", examNumber=" + examNumber + ", paperNumber=" + paperNumber
                    + ", pageIndex=" + pageIndex, e);
            throw new StatusException("原图上传失败", e);
        } finally {
            concurrentService.getReadWriteLock(LockType.STUDENT + "-" + student.getId()).writeLock().unlock();
        }
    }

    @Transactional
    @Override
    public SliceUploadVo sliceUpload(Long batchId, String examNumber, Integer paperNumber, Integer pageIndex,
            Integer index, MultipartFile multipartFile, String md5) {
        BatchEntity batch = batchService.getById(batchId);
        if (batch == null) {
            throw new ParameterException("批次信息未找到");
        }
        batchService.checkBatchStatus(batch);
        StudentEntity student = studentService.findByExamAndExamNumber(batch.getExamId(), examNumber);
        if (student == null) {
            throw new ParameterException("考生信息未找到");
        }
        concurrentService.getReadWriteLock(LockType.STUDENT + "-" + student.getId()).writeLock().lock();
        try {
            String path = fileService
                    .uploadSlice(multipartFile.getInputStream(), md5, batch.getExamId(), batchId, examNumber,
                            paperNumber, pageIndex, index);
            filePropertyService.save(batch.getExamId(), path, md5);
            SliceUploadVo vo = new SliceUploadVo();
            vo.setUri(path);
            vo.setUpdateTime(new Date().getTime());
            return vo;
        } catch (Exception e) {
            log.error("裁切图上传失败，batchId=" + batch + ", examNumber=" + examNumber + ", paperNumber=" + paperNumber
                    + ", pageIndex=" + pageIndex + ", index=" + index, e);
            throw new StatusException("裁切图上传失败", e);
        } finally {
            concurrentService.getReadWriteLock(LockType.STUDENT + "-" + student.getId()).writeLock().unlock();
        }
    }

    @Override
    public Integer findCountByBatch(Long batchId) {
        QueryWrapper<PaperEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<PaperEntity> lw = wrapper.lambda();
        lw.eq(PaperEntity::getBatchId, batchId);
        return baseMapper.selectCount(wrapper);
    }

    @Override
    public Integer findStudentCountByBatch(Long batchId) {
        return baseMapper.findStudentCountByBatch(batchId);
    }

    @Override
    public List<ScannerWorkloadVo> workload(WorkloadDomain domain) {
        Long examId = domain.getExamId();
        Long startTime = domain.getStartTime();
        Long endTime = domain.getEndTime();
        if (examId == null) {
            throw new ParameterException("examId不能为空");
        }
        return baseMapper.workload(examId, startTime, endTime);
    }

    @Override
    public List<PaperEntity> findByBatch(Long id) {
        QueryWrapper<PaperEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<PaperEntity> lw = wrapper.lambda();
        lw.eq(PaperEntity::getBatchId, id);
        wrapper.orderByAsc("number");
        return this.list(wrapper);
    }

    @Override
    public List<PaperEntity> listByMismatchQuery(List<MismatchQueryVo> paramList) {
        return baseMapper.listByMismatchQuery(paramList);
    }

    @Override
    public List<StudentPaperVo> listByStudentIds(List<Long> paramList) {
        return baseMapper.listByStudentIds(paramList);
    }

    @Transactional
    @Override
    public UriVo adminSliceUpload(Long paperId, Integer pageIndex, Integer index, MultipartFile multipartFile,
            String md5) {
        PaperEntity paper = this.getById(paperId);
        if (paper == null) {
            throw new ParameterException("paper信息未找到");
        }
        PaperPageEntity page = paperPageService.findPaperIdAndIndex(paperId, pageIndex);
        if (page == null) {
            throw new ParameterException("page信息未找到");
        }
        if (index <= 0) {
            throw new ParameterException("index必须大于0");
        }
        //TODO 此处没有student信息，无法使用studentpaper上的paperNumber
        concurrentService.getReadWriteLock(LockType.STUDENT + "-" + paper.getStudentId()).writeLock().lock();
        try {
            String path = fileService
                    .adminUploadSlice(multipartFile.getInputStream(), md5, paper.getExamId(), paper.getExamNumber(),
                            paper.getNumber(), pageIndex, index);
            filePropertyService.save(paper.getExamId(), path, md5);
            UriVo vo = new UriVo();
            vo.setUri(path);
            return vo;
        } catch (Exception e) {
            log.error("裁切图上传失败，paperId=" + paperId + ", pageIndex=" + pageIndex + ", index=" + index, e);
            throw new StatusException("裁切图上传失败", e);
        } finally {
            concurrentService.getReadWriteLock(LockType.STUDENT + "-" + paper.getStudentId()).writeLock().unlock();
        }
    }

    @Transactional
    @Override
    public PageDeleteVo pageDelete(User user, PageDeleteDomain domain) {
        Long examId = domain.getExamId();
        String examNumber = domain.getExamNumber();
        Integer paperNumber = domain.getPaperNumber();
        if (examId == null) {
            throw new ParameterException("examId不能为空");
        }
        if (StringUtils.isBlank(examNumber)) {
            throw new ParameterException("examNumber不能为空");
        }
        if (paperNumber == null) {
            throw new ParameterException("paperNumber不能为空");
        }
        StudentEntity student = studentService.findByExamAndExamNumber(examId, examNumber);
        if (student == null) {
            throw new ParameterException("考生信息未找到");
        }
        concurrentService.getReadWriteLock(LockType.STUDENT + "-" + student.getId()).writeLock().lock();
        try {
            List<StudentPaperEntity> list = studentPaperService.findByStudentId(student.getId());
            if (CollectionUtils.isEmpty(list)) {
                throw new ParameterException("未找到该张扫描结果");
            }
            if (list.size() > 1 && paperNumber == 1) {
                throw new StatusException("需先删除其他张之后才能删除第一张");
            }
            StudentPaperEntity sp = getByNumber(list, paperNumber);
            if (sp == null) {
                throw new ParameterException("未找到该张扫描结果");
            }
            studentPaperService.removeByStudentIdAndPaperNumber(student.getId(), paperNumber);
            if (paperNumber == 1) {
                student.setAbsentSuspect(false);
                student.setAssigned(false);
                student.setOmrAbsent(false);
                student.setIncomplete(false);
                student.setQuestionFilled(false);
                student.setStatus(ScanStatus.UNEXIST);
            } else {
                student.setIncomplete(true);
            }
            student.setFileUploadStatus(UploadStatus.WAITING_UPLOAD);
            student.setDataUploadStatus(UploadStatus.WAITING_UPLOAD);
            studentService.saveOrUpdate(student);
            omrTaskService.deleteByStudentId(examId, user, student.getId());
            PageDeleteVo vo = new PageDeleteVo();
            vo.setPaperCount(list.size() - 1);
            vo.setUpdateTime(new Date().getTime());
            return vo;
        } finally {
            concurrentService.getReadWriteLock(LockType.STUDENT + "-" + student.getId()).writeLock().unlock();
        }
    }

    private StudentPaperEntity getByNumber(List<StudentPaperEntity> list, Integer paperNumber) {
        for (StudentPaperEntity s : list) {
            if (s.getPaperNumber().intValue() == paperNumber.intValue()) {
                return s;
            }
        }
        return null;
    }

    @Transactional
    @Override
    public AnswerDeleteVo answerDelete(User user, AnswerDeleteDomain domain) {
        Long examId = domain.getExamId();
        String examNumber = domain.getExamNumber();
        if (examId == null) {
            throw new ParameterException("examId不能为空");
        }
        if (StringUtils.isBlank(examNumber)) {
            throw new ParameterException("examNumber不能为空");
        }
        StudentEntity student = studentService.findByExamAndExamNumber(examId, examNumber);
        if (student == null) {
            throw new ParameterException("考生信息未找到");
        }
        concurrentService.getReadWriteLock(LockType.STUDENT + "-" + student.getId()).writeLock().lock();
        try {
            List<StudentPaperEntity> list = studentPaperService.findByStudentId(student.getId());
            if (CollectionUtils.isEmpty(list)) {
                throw new ParameterException("未找到扫描结果");
            }
            studentPaperService.removeByStudentId(student.getId());
            student.setAbsentSuspect(false);
            student.setAssigned(false);
            student.setOmrAbsent(false);
            student.setIncomplete(false);
            student.setQuestionFilled(false);
            student.setCardNumber(null);
            student.setStatus(ScanStatus.UNEXIST);
            student.setFileUploadStatus(UploadStatus.WAITING_UPLOAD);
            student.setDataUploadStatus(UploadStatus.WAITING_UPLOAD);
            studentService.saveOrUpdate(student);
            omrTaskService.deleteByStudentId(examId, user, student.getId());
            AnswerDeleteVo vo = new AnswerDeleteVo();
            vo.setStatus(ScanStatus.UNEXIST);
            vo.setUpdateTime(new Date().getTime());
            return vo;
        } finally {
            concurrentService.getReadWriteLock(LockType.STUDENT + "-" + student.getId()).writeLock().unlock();
        }
    }

    @Transactional
    @Override
    public UriVo adminSliceUpdate(Long paperId, Integer pageIndex, Integer index, MultipartFile multipartFile,
            String md5) {
        PaperEntity paper = this.getById(paperId);
        if (paper == null) {
            throw new ParameterException("paper信息未找到");
        }
        PaperPageEntity page = paperPageService.findPaperIdAndIndex(paperId, pageIndex);
        if (page == null) {
            throw new ParameterException("page信息未找到");
        }
        if (index <= 0) {
            throw new ParameterException("index必须大于0");
        }
        if (CollectionUtils.isEmpty(page.getSlicePath()) || page.getSlicePath().size() < index || StringUtils
                .isBlank(page.getSlicePath().get(index - 1))) {
            throw new ParameterException("未找到对应的裁切图");
        }
        StudentPaperEntity sp = studentPaperService.findByPaperId(paperId);
        if(sp ==null) {
            throw new ParameterException("paper未绑定考生");
        }
        concurrentService.getReadWriteLock(LockType.STUDENT + "-" + sp.getStudentId()).writeLock().lock();
        try {
            String path = fileService
                    .adminUploadSlice(multipartFile.getInputStream(), md5, paper.getExamId(), paper.getExamNumber(),
                            sp.getPaperNumber(), pageIndex, index);
            filePropertyService.save(paper.getExamId(), path, md5);
            page.getSlicePath().set(index - 1, path);
            paperPageService.updateByMultiId(page);
            //更新考生文件上传状态
            studentService.updateFileUploadStatus(sp.getStudentId(), UploadStatus.WAITING_UPLOAD);
            UriVo vo = new UriVo();
            vo.setUri(path);
            return vo;
        } catch (Exception e) {
            log.error("裁切图更新失败，paperId=" + paperId + ", pageIndex=" + pageIndex + ", index=" + index, e);
            throw new StatusException("裁切图更新失败", e);
        } finally {
            concurrentService.getReadWriteLock(LockType.STUDENT + "-" + sp.getStudentId()).writeLock().unlock();
        }
    }

    @Override
    public List<PaperEntity> findAssigneByBatch(Long batchId) {
        return baseMapper.findAssigneByBatch(batchId);
    }

    @Override
    public int getCountByExamAndCardNumber(Long examId, Integer number) {
        QueryWrapper<PaperEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<PaperEntity> lw = wrapper.lambda();
        lw.eq(PaperEntity::getExamId, examId);
        lw.eq(PaperEntity::getCardNumber, number);
        return this.count(wrapper);
    }
}
