package cn.com.qmth.scancloud.service.impl;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonObject;
import com.google.gson.JsonSyntaxException;
import com.qmth.boot.core.exception.ParameterException;

import cn.com.qmth.scancloud.dao.PaperStructureDao;
import cn.com.qmth.scancloud.entity.PaperStructureEntity;
import cn.com.qmth.scancloud.enums.OmrField;
import cn.com.qmth.scancloud.service.PaperStructureService;
import cn.com.qmth.scancloud.util.FileUtil;
import cn.com.qmth.scancloud.vo.paperstructureinfo.PaperStructure;
import cn.com.qmth.scancloud.vo.paperstructureinfo.PaperStructureVo;

@Service
public class PaperStructureServiceImpl extends ServiceImpl<PaperStructureDao, PaperStructureEntity>
		implements PaperStructureService {

	@Autowired
	private PaperStructureService paperStructureService;

	@Transactional
	@Override
	public void updateByAnswerCard(InputStream in, Long examId, Integer cardNumber) {
		if (examId == null) {
			throw new ParameterException("考试ID不能为空");
		}
		if (cardNumber == null) {
			throw new ParameterException("卡格式编号不能为空");
		}
		String cardJson = FileUtil.readFileContent(in);
		paperStructureService.deleteByAnswerCard(examId, cardNumber);
		if (StringUtils.isBlank(cardJson)) {
			return;
		}
		Gson gs = new Gson();
		JsonObject js;
		try {
			js = gs.fromJson(cardJson, JsonObject.class);
		} catch (JsonSyntaxException e) {
			throw new ParameterException("卡格式信息解析失败", e);
		}
		List<PaperStructureEntity> pss = parse(js, examId, cardNumber);
		if (CollectionUtils.isNotEmpty(pss)) {
			this.saveOrUpdateBatch(pss);
		}
	}

	@Transactional
	@Override
	@CacheEvict(value = "PAPER_STRUCTURE", key = "#examId-#cardNumber", beforeInvocation = true)
	public void deleteByAnswerCard(Long examId, Integer cardNumber) {
		if (examId == null) {
			throw new ParameterException("考试ID不能为空");
		}
		if (cardNumber == null) {
			throw new ParameterException("卡格式编号不能为空");
		}
		QueryWrapper<PaperStructureEntity> wrapper = new QueryWrapper<>();
		LambdaQueryWrapper<PaperStructureEntity> lw = wrapper.lambda();
		lw.eq(PaperStructureEntity::getExamId, examId);
		lw.eq(PaperStructureEntity::getCardNumber, cardNumber);
		this.baseMapper.delete(wrapper);
	}

	private List<PaperStructureEntity> parse(JsonObject js, Long examId, Integer cardNumber) {
		List<PaperStructureEntity> pss = new ArrayList<>();
		JsonArray pages = js.getAsJsonArray("pages");
		if (pages == null || pages.size() == 0) {
			return pss;
		}
		int paperNumber = 0;
		int pageIndex = 0;
		for (int i = 0; i < pages.size(); i++) {
			paperNumber = (i + 2) / 2;
			int mod = (i + 1) % 2;
			pageIndex = (mod == 0 ? 2 : mod);
			JsonObject page = pages.get(i).getAsJsonObject();
			if (page != null) {
				JsonObject exchange = page.getAsJsonObject("exchange");
				if (exchange != null) {
					JsonArray fillAreas = exchange.getAsJsonArray("fill_area");
					if (fillAreas != null && fillAreas.size() != 0) {
						int questionNumber = 0;
						int selectiveNumber = 0;
						List<JsonObject> sortFillAreas = sort(fillAreas);
						for (int j = 0; j < sortFillAreas.size(); j++) {
							JsonObject fillArea = sortFillAreas.get(j);
							String field = fillArea.get("field").getAsString();
							if ("question".equals(field) || "selective".equals(field)) {
								boolean single = fillArea.get("single").getAsBoolean();
								if ("question".equals(field)) {
									JsonArray items = fillArea.getAsJsonArray("items");
									for (int k = 0; k < items.size(); k++) {
										questionNumber++;
										PaperStructureEntity ps = new PaperStructureEntity();
										pss.add(ps);
										ps.setExamId(examId);
										ps.setCardNumber(cardNumber);
										ps.setPaperNumber(paperNumber);
										ps.setPageIndex(pageIndex);
										ps.setField(OmrField.QUESTION);
										ps.setSingle(single);
										ps.setQuestionNumber(questionNumber);
									}
								} else {
									JsonArray items = fillArea.getAsJsonArray("items");
									for (int k = 0; k < items.size(); k++) {
										selectiveNumber++;
										PaperStructureEntity ps = new PaperStructureEntity();
										pss.add(ps);
										ps.setExamId(examId);
										ps.setCardNumber(cardNumber);
										ps.setPaperNumber(paperNumber);
										ps.setPageIndex(pageIndex);
										ps.setField(OmrField.SELECTIVE);
										ps.setSingle(single);
										ps.setQuestionNumber(selectiveNumber);
									}
								}
							}
						}
					}
				}
			}
		}
		return pss;
	}

	private List<JsonObject> sort(JsonArray fillAreas) {
		List<JsonObject> list = new ArrayList<>();
		for (int j = 0; j < fillAreas.size(); j++) {
			list.add(fillAreas.get(j).getAsJsonObject());
		}
		list.sort(new Comparator<JsonObject>() {

			@Override
			public int compare(JsonObject o1, JsonObject o2) {
				int indx1 = o1.get("index").getAsInt();
				int indx2 = o2.get("index").getAsInt();
				if (indx1 < indx2) {
					return -1;
				} else if (indx1 > indx2) {
					return 1;
				}
				return 0;
			}
		});
		return list;
	}

	@Override
	@Cacheable(value = "PAPER_STRUCTURE", key = "#examId-#cardNumber", unless = "#result == null")
	public PaperStructureVo getByAnswerCard(Long examId, Integer cardNumber) {
		if (examId == null) {
			throw new ParameterException("考试ID不能为空");
		}
		if (cardNumber == null) {
			throw new ParameterException("卡格式编号不能为空");
		}
		QueryWrapper<PaperStructureEntity> wrapper = new QueryWrapper<>();
		LambdaQueryWrapper<PaperStructureEntity> lw = wrapper.lambda();
		lw.eq(PaperStructureEntity::getExamId, examId);
		lw.eq(PaperStructureEntity::getCardNumber, cardNumber);
		List<PaperStructureEntity> list = this.list(wrapper);
		if (CollectionUtils.isEmpty(list)) {
			return null;
		}

		PaperStructureVo vo = new PaperStructureVo();
		List<PaperStructure> selective = new ArrayList<PaperStructure>();
		List<PaperStructure> question = new ArrayList<PaperStructure>();
		vo.setQuestion(question);
		vo.setSelective(selective);
		for (PaperStructureEntity e : list) {
			if (OmrField.QUESTION.equals(e.getField())) {
				question.add(PaperStructure.of(e));
			} else if (OmrField.SELECTIVE.equals(e.getField())) {
				question.add(PaperStructure.of(e));
			}
		}
		return vo;
	}

	@Override
	public PaperStructure getPaperStructure(Long examId, Integer cardNumber, Integer paperNumber, Integer pageIndex,
			Integer questionNumber, OmrField omrField) {
		if (examId == null) {
			throw new ParameterException("考试ID不能为空");
		}
		if (cardNumber == null) {
			throw new ParameterException("卡格式编号不能为空");
		}
		if (paperNumber == null) {
			throw new ParameterException("paperNumber不能为空");
		}
		if (pageIndex == null) {
			throw new ParameterException("pageIndex不能为空");
		}
		if (questionNumber == null) {
			throw new ParameterException("questionNumber不能为空");
		}
		if (omrField == null) {
			throw new ParameterException("omrField不能为空");
		}
		PaperStructureVo vo = paperStructureService.getByAnswerCard(examId, cardNumber);
		if (vo == null) {
			return null;
		}
		List<PaperStructure> question = null;
		if (OmrField.QUESTION.equals(omrField)) {
			question = vo.getQuestion();
		} else if (OmrField.SELECTIVE.equals(omrField)) {
			question = vo.getSelective();
		}
		if (CollectionUtils.isEmpty(question)) {
			return null;
		}
		for (PaperStructure ps : question) {
			if (paperNumber.intValue() == ps.getPaperNumber().intValue()
					&& pageIndex.intValue() == ps.getPageIndex().intValue()
					&& questionNumber.intValue() == ps.getQuestionNumber().intValue()) {
				return ps;
			}
		}
		return null;
	}
}
