package cn.com.qmth.scancloud.service.impl;

import java.util.List;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.jeffreyning.mybatisplus.service.MppServiceImpl;

import cn.com.qmth.scancloud.dao.StudentPaperDao;
import cn.com.qmth.scancloud.entity.StudentPaperEntity;
import cn.com.qmth.scancloud.service.StudentPaperService;
import cn.com.qmth.scancloud.vo.StudentPaperCountVo;

@Service
public class StudentPaperServiceImpl extends MppServiceImpl<StudentPaperDao, StudentPaperEntity>
        implements StudentPaperService {

    @Transactional
    @Override
    public void update(Long studentId, Integer paperNumber, Long paperId) {
        QueryWrapper<StudentPaperEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentPaperEntity> lw = wrapper.lambda();
        lw.eq(StudentPaperEntity::getStudentId, studentId);
        lw.eq(StudentPaperEntity::getPaperNumber, paperNumber);
        StudentPaperEntity e = baseMapper.selectOne(wrapper);
        if (e == null) {
            e = new StudentPaperEntity();
            e.setStudentId(studentId);
            e.setPaperNumber(paperNumber);
        }
        e.setPaperId(paperId);
        this.saveOrUpdateByMultiId(e);
    }

    @Override
    public List<StudentPaperEntity> findPaperIdByExamId(Long examId) {
        return this.baseMapper.findPaperIdByExamId(examId);
    }

    @Override
    public StudentPaperEntity findByStudentIdAndNumber(Long studentId, Integer number) {
        QueryWrapper<StudentPaperEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentPaperEntity> lw = wrapper.lambda();
        lw.eq(StudentPaperEntity::getStudentId, studentId);
        lw.eq(StudentPaperEntity::getPaperNumber, number);
        return baseMapper.selectOne(wrapper);
    }

    @Override
    public List<StudentPaperEntity> findByStudentId(Long studentId) {
        QueryWrapper<StudentPaperEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentPaperEntity> lw = wrapper.lambda();
        lw.eq(StudentPaperEntity::getStudentId, studentId);
        wrapper.orderByAsc("paper_number");
        return this.list(wrapper);
    }

    @Override
    public List<StudentPaperCountVo> findStudentPaperCount(List<Long> studentIds) {
        return baseMapper.findStudentPaperCount(studentIds);
    }

	@Override
	public StudentPaperEntity findByStudentIdAndPaperId(Long studentId, Long paperId) {
		QueryWrapper<StudentPaperEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentPaperEntity> lw = wrapper.lambda();
        lw.eq(StudentPaperEntity::getStudentId, studentId);
        lw.eq(StudentPaperEntity::getPaperId, paperId);
        return baseMapper.selectOne(wrapper);
	}

	@Transactional
	@Override
	public void removeByStudentId(Long studentId) {
		QueryWrapper<StudentPaperEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentPaperEntity> lw = wrapper.lambda();
        lw.eq(StudentPaperEntity::getStudentId, studentId);
        this.baseMapper.delete(wrapper);
	}

	@Transactional
	@Override
	public void removeByStudentIdAndPaperNumber(Long studentId, Integer paperNumber) {
		QueryWrapper<StudentPaperEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentPaperEntity> lw = wrapper.lambda();
        lw.eq(StudentPaperEntity::getStudentId, studentId);
        lw.eq(StudentPaperEntity::getPaperNumber, paperNumber);
        this.baseMapper.delete(wrapper);
	}

	@Override
	public StudentPaperEntity findByPaperId(Long paperId) {
		QueryWrapper<StudentPaperEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentPaperEntity> lw = wrapper.lambda();
        lw.eq(StudentPaperEntity::getPaperId, paperId);
        return baseMapper.selectOne(wrapper);
	}

}
