package cn.com.qmth.scancloud.service.impl;

import cn.com.qmth.scancloud.bean.AbsentQueryDomain;
import cn.com.qmth.scancloud.bean.AnswerQueryDomain;
import cn.com.qmth.scancloud.bean.ImportStudentDomain;
import cn.com.qmth.scancloud.dao.StudentDao;
import cn.com.qmth.scancloud.entity.*;
import cn.com.qmth.scancloud.enums.GroupType;
import cn.com.qmth.scancloud.enums.ScanStatus;
import cn.com.qmth.scancloud.enums.UploadStatus;
import cn.com.qmth.scancloud.exception.ParameterExceptions;
import cn.com.qmth.scancloud.service.*;
import cn.com.qmth.scancloud.util.BatchGetDataUtil;
import cn.com.qmth.scancloud.util.BatchSetDataUtil;
import cn.com.qmth.scancloud.util.PageUtil;
import cn.com.qmth.scancloud.vo.*;
import cn.com.qmth.scancloud.vo.answerquery.AnswerPageVo;
import cn.com.qmth.scancloud.vo.answerquery.AnswerPaperVo;
import cn.com.qmth.scancloud.vo.answerquery.AnswerQueryVo;
import cn.com.qmth.scancloud.vo.answerquery.StudentPaperVo;
import cn.com.qmth.scancloud.vo.student.StudentQuery;
import cn.com.qmth.scancloud.vo.student.StudentVo;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.core.collection.PageResult;
import com.qmth.boot.core.exception.ParameterException;
import com.qmth.boot.tools.excel.ExcelReader;
import com.qmth.boot.tools.excel.enums.ExcelType;
import com.qmth.boot.tools.excel.model.DataMap;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class StudentServiceImpl extends ServiceImpl<StudentDao, StudentEntity> implements StudentService {

    @Autowired
    private PaperService paperService;

    @Autowired
    private PaperPageService paperPageService;

    @Autowired
    private StudentPaperService studentPaperService;

    @Autowired
    private ExamService examService;

    @Autowired
    private SubjectService subjectService;

    @Autowired
    private BatchService batchService;

    @Override
    public List<String> listCampusByExamId(Long examId) {
        return baseMapper.listCampusByExamId(examId);
    }

    @Override
    public List<String> listSiteByExamId(Long examId) {
        return baseMapper.listSiteByExamId(examId);
    }

    @Override
    public StudentVo findOne(StudentQuery query) {
        Long examId = query.getExamId();
        String examNumber = query.getExamNumber();
        String studentCode = query.getStudentCode();
        String subjectCode = query.getSubjectCode();
        if (examId == null) {
            throw new ParameterException("examId 不能为空");
        }
        if (StringUtils.isBlank(examNumber) && StringUtils.isBlank(studentCode) && StringUtils.isBlank(subjectCode)) {
            throw new ParameterException("examNumber studentCode subjectCode不能都为空");
        }
        if (StringUtils.isBlank(examNumber) && (StringUtils.isBlank(studentCode) || StringUtils.isBlank(subjectCode))) {
            throw new ParameterException("examNumber为空时 studentCode subjectCode都不能为空");
        }
        QueryWrapper<StudentEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(StudentEntity::getExamId, examId);
        if (StringUtils.isNotBlank(examNumber)) {
            queryWrapper.lambda().eq(StudentEntity::getExamNumber, examNumber);
            StudentEntity s = baseMapper.selectOne(queryWrapper);
            if (s == null) {
                throw new ParameterException("未找到考生");
            }
            StudentVo vo = StudentVo.of(s);
            SubjectEntity sub = subjectService.findByExamIdAndCode(examId, s.getSubjectCode());
            vo.setSubjectName(sub.getName());
            return vo;
        } else if (StringUtils.isNotBlank(studentCode) && StringUtils.isNotBlank(subjectCode)) {
            queryWrapper.lambda().eq(StudentEntity::getStudentCode, studentCode);
            queryWrapper.lambda().eq(StudentEntity::getSubjectCode, subjectCode);
            StudentEntity s = baseMapper.selectOne(queryWrapper);
            if (s == null) {
                throw new ParameterException("未找到考生");
            }
            StudentVo vo = StudentVo.of(s);
            SubjectEntity sub = subjectService.findByExamIdAndCode(examId, s.getSubjectCode());
            vo.setSubjectName(sub.getName());
            return vo;
        } else {
            return null;
        }
    }

    @Override
    public int getCountByExam(Long examId) {
        QueryWrapper<StudentEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentEntity> lw = wrapper.lambda();
        lw.eq(StudentEntity::getExamId, examId);
        return this.count(wrapper);
    }

    @Override
    public int getPackageCountByExam(Long examId) {
        return baseMapper.getPackageCountByExam(examId);
    }

    @Override
    public int getCountByExamAndScanStatus(Long examId, ScanStatus status) {
        QueryWrapper<StudentEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentEntity> lw = wrapper.lambda();
        lw.eq(StudentEntity::getExamId, examId);
        lw.eq(StudentEntity::getStatus, status);
        return this.count(wrapper);
    }

    @Override
    public List<StudentVo> packageList(StudentQuery query) {
        List<StudentVo> ret = new ArrayList<>();
        StudentVo one = findOne(query);
        if (one != null) {
            List<StudentEntity> list = findByExamAndPackage(query.getExamId(), one.getPackageCode());
            if (CollectionUtils.isNotEmpty(list)) {
                for (StudentEntity e : list) {
                    StudentVo vo = StudentVo.of(e);
                    vo.setSubjectName(one.getSubjectName());
                    ret.add(vo);
                }
            }
        }
        return ret;
    }

    @Override
    public List<StudentEntity> findByExamAndPackage(Long examId, String packageCode) {
        if (examId == null) {
            throw new ParameterException("examId 不能为空");
        }
        if (StringUtils.isBlank(packageCode)) {
            throw new ParameterException("packageCode不能为空");
        }
        QueryWrapper<StudentEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(StudentEntity::getExamId, examId);
        queryWrapper.lambda().eq(StudentEntity::getPackageCode, packageCode);
        return this.list(queryWrapper);
    }

    @Override
    public StudentEntity findByExamAndExamNumber(Long examId, String examNumber) {
        if (examId == null) {
            throw new ParameterException("examId 不能为空");
        }
        if (StringUtils.isBlank(examNumber)) {
            throw new ParameterException("examNumber不能为空");
        }
        QueryWrapper<StudentEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(StudentEntity::getExamId, examId);
        queryWrapper.lambda().eq(StudentEntity::getExamNumber, examNumber);
        return baseMapper.selectOne(queryWrapper);
    }

    @Override
    public StudentEntity findByStudentCodeAndSubjectCode(Long examId,String studentCode, String subjectCode) {
        if (StringUtils.isBlank(studentCode)) {
            throw new ParameterException("考生编号不能为空");
        }
        if (StringUtils.isBlank(subjectCode)) {
            throw new ParameterException("科目代码不能为空");
        }
        QueryWrapper<StudentEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(StudentEntity::getExamId, examId);
        queryWrapper.lambda().eq(StudentEntity::getStudentCode, studentCode);
        queryWrapper.lambda().eq(StudentEntity::getSubjectCode, subjectCode);
        return baseMapper.selectOne(queryWrapper);
    }

    @Override
    public int getAssignedCountByExam(Long examId) {
        QueryWrapper<StudentEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentEntity> lw = wrapper.lambda();
        lw.eq(StudentEntity::getExamId, examId);
        lw.eq(StudentEntity::getAssigned, true);
        return this.count(wrapper);
    }

    @Override
    public int getAbsentSuspectCountByExam(Long examId) {
        QueryWrapper<StudentEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentEntity> lw = wrapper.lambda();
        lw.eq(StudentEntity::getExamId, examId);
        lw.eq(StudentEntity::getAbsentSuspect, true);
        return this.count(wrapper);
    }

    @Override
    public int getIncompleteCountByExam(Long examId) {
        QueryWrapper<StudentEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentEntity> lw = wrapper.lambda();
        lw.eq(StudentEntity::getExamId, examId);
        lw.eq(StudentEntity::getIncomplete, true);
        return this.count(wrapper);
    }

    @Override
    public AbsentInfoVo absentInfo(Long examId) {
        AbsentInfoVo vo = new AbsentInfoVo();
        vo.setTotalCount(totalCount(examId));
        vo.setScannedCount(countByStatus(examId, ScanStatus.SCANNED));
        vo.setUnexistCount(countByStatus(examId, ScanStatus.UNEXIST));
        vo.setManualAbsentCount(countByStatus(examId, ScanStatus.MANUAL_ABSENT));
        vo.setOmrAbsentCount(getOmrAbsentCountByExam(examId));
        vo.setAbsentSuspectCount(countByAbsentSuspect(examId));
        return vo;
    }

    private Integer totalCount(Long examId) {
        QueryWrapper<StudentEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentEntity> lw = wrapper.lambda();
        lw.eq(StudentEntity::getExamId, examId);
        return this.count(wrapper);
    }

    private Integer countByStatus(Long examId, ScanStatus st) {
        QueryWrapper<StudentEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentEntity> lw = wrapper.lambda();
        lw.eq(StudentEntity::getExamId, examId);
        lw.eq(StudentEntity::getStatus, st);
        return this.count(wrapper);
    }

    private Integer countByAbsentSuspect(Long examId) {
        QueryWrapper<StudentEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentEntity> lw = wrapper.lambda();
        lw.eq(StudentEntity::getExamId, examId);
        lw.eq(StudentEntity::getAbsentSuspect, true);
        return this.count(wrapper);
    }

    private Integer countByIncomplete(Long examId) {
        QueryWrapper<StudentEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentEntity> lw = wrapper.lambda();
        lw.eq(StudentEntity::getExamId, examId);
        lw.eq(StudentEntity::getIncomplete, true);
        return this.count(wrapper);
    }

    @Override
    public PageResult<AbsentQueryVo> absentQuery(AbsentQueryDomain query) {
        if (query.getExamId() == null) {
            throw new ParameterException("examId 不能为空");
        }
        if (query.getGroupType() == null) {
            throw new ParameterException("groupType 不能为空");
        }
        if (CollectionUtils.isEmpty(query.getStatus())) {
            throw new ParameterException("status 不能为空");
        }
        String groupNameCol = null;
        if (GroupType.PACKAGE.equals(query.getGroupType())) {
            groupNameCol = "package_code";
        }
        if (GroupType.SUBJECT.equals(query.getGroupType())) {
            groupNameCol = "subject_code";
        }
        if (GroupType.CAMPUS.equals(query.getGroupType())) {
            groupNameCol = "campus_name";
        }
        if (GroupType.EXAM_SITE.equals(query.getGroupType())) {
            groupNameCol = "exam_site";
        }
        if (GroupType.EXAM_ROOM.equals(query.getGroupType())) {
            groupNameCol = "exam_room";
        }
        if (GroupType.STUDENT_CODE.equals(query.getGroupType())) {
            groupNameCol = "student_code";
        }
        IPage<AbsentQueryVo> iPage = baseMapper
                .absentQueryPage(new Page<AbsentQueryVo>(query.getPageNumber(), query.getPageSize()), groupNameCol,
                        query);
        return PageUtil.of(iPage);
    }

    @Transactional
    @Override
    public AbsentManualImportVo absentManualImport(Long examId, MultipartFile multipartFile) {
        List<DataMap> lineList = null;
        try {
            lineList = ExcelReader.create(ExcelType.XLSX, multipartFile.getInputStream(), 0).getDataMapList();
        } catch (IOException e) {
            throw new ParameterException("Excel解析出错", e);
        } catch (Exception e) {
            throw new ParameterException("Excel解析出错", e);
        }
        if (CollectionUtils.isEmpty(lineList)) {
            throw new ParameterException("Excel无内容");
        }

        //        if (10001 < lineList.size()) {
        //            throw new ParameterException("数据行数不能超过10000");
        //        }

        int updateCount = 0;
        int ignoreCount = 0;
        int suspectCount = 0;
        for (int i = 0; i < lineList.size(); i++) {
            DataMap line = lineList.get(i);
            String studentCode = trimAndNullIfBlank(line.getValue(0));
            String subjectCode = trimAndNullIfBlank(line.getValue(1));
            if (StringUtils.isBlank(studentCode)) {
                throw new ParameterException("考生编号不能为空");
            }
            if (StringUtils.isBlank(subjectCode)) {
                throw new ParameterException("科目代码不能为空");
            }
            StudentEntity student = findByStudentCodeAndSubjectCode(examId,studentCode, subjectCode);
            if (student == null) {
                throw new ParameterException("考生信息未找到:" + studentCode + ":" + subjectCode);
            }
            if (ScanStatus.MANUAL_ABSENT.equals(student.getStatus())) {
                ignoreCount++;
            } else if (ScanStatus.SCANNED.equals(student.getStatus())) {
                student.setAbsentSuspect(true);
                this.saveOrUpdate(student);
                suspectCount++;
            } else {
                student.setStatus(ScanStatus.MANUAL_ABSENT);
                student.setDataUploadStatus(UploadStatus.WAITING_UPLOAD);
                this.saveOrUpdate(student);
                updateCount++;
            }

        }
        AbsentManualImportVo vo = new AbsentManualImportVo();
        vo.setIgnoreCount(ignoreCount);
        vo.setUpdateCount(updateCount);
        vo.setSuspectCount(suspectCount);
        return vo;
    }

    private String trimAndNullIfBlank(String s) {
        if (StringUtils.isBlank(s)) {
            return null;
        }
        return s.trim();
    }

    @Transactional
    @Override
    public UpdateTimeVo absentManualUpdate(Long examId, String examNumber) {
        StudentEntity student = findByExamAndExamNumber(examId, examNumber);
        if (student == null) {
            throw new ParameterException("准考证号未找到:" + examNumber);
        }
        student.setStatus(ScanStatus.MANUAL_ABSENT);
        student.setDataUploadStatus(UploadStatus.WAITING_UPLOAD);
        this.saveOrUpdate(student);
        UpdateTimeVo vo = new UpdateTimeVo();
        vo.setUpdateTime(System.currentTimeMillis());
        return vo;
    }

    @Transactional
    @Override
    public UpdateTimeVo absentSuspectRemove(Long examId, String examNumber) {
        StudentEntity student = findByExamAndExamNumber(examId, examNumber);
        if (student == null) {
            throw new ParameterException("准考证号未找到:" + examNumber);
        }
        student.setAbsentSuspect(false);
        student.setDataUploadStatus(UploadStatus.WAITING_UPLOAD);
        this.saveOrUpdate(student);
        UpdateTimeVo vo = new UpdateTimeVo();
        vo.setUpdateTime(System.currentTimeMillis());
        return vo;
    }

    @Override
    public ScanAnswerInfoVo scanAnswerInfo(Long examId) {
        ScanAnswerInfoVo vo = new ScanAnswerInfoVo();
        vo.setTotalCount(totalCount(examId));
        vo.setScannedCount(countByStatus(examId, ScanStatus.SCANNED));
        vo.setUnexistCount(countByStatus(examId, ScanStatus.UNEXIST));
        vo.setManualAbsentCount(countByStatus(examId, ScanStatus.MANUAL_ABSENT));
        vo.setOmrAbsentCount(getOmrAbsentCountByExam(examId));
        vo.setAbsentSuspectCount(countByAbsentSuspect(examId));
        vo.setIncompleteCount(countByIncomplete(examId));
        return vo;
    }

    @Override
    public PageResult<AnswerQueryVo> query(AnswerQueryDomain query) {
        if (query.getExamId() == null) {
            throw new ParameterException("examId 不能为空");
        }
        //查询考生分页信息
        IPage<AnswerQueryVo> iPage = baseMapper
                .queryPage(new Page<AnswerQueryVo>(query.getPageNumber(), query.getPageSize()), query);
        if (CollectionUtils.isNotEmpty(iPage.getRecords())) {
            Map<Long, AnswerQueryVo> map = new HashMap<>();

            for (AnswerQueryVo vo : iPage.getRecords()) {
                List<AnswerPaperVo> papers = new ArrayList<>();
                vo.setPapers(papers);
                if (vo.getCardPaperCount() != null) {
                    for (int i = 1; i <= vo.getCardPaperCount(); i++) {
                        AnswerPaperVo pv = new AnswerPaperVo();
                        pv.setNumber(i);
                        papers.add(pv);
                    }
                }
                map.put(vo.getId(), vo);
            }
            //根据考生id查找绑定paper
            List<Long> studentIds = iPage.getRecords().stream().map(p -> p.getId()).collect(Collectors.toList());
            List<StudentPaperVo> paperList = new BatchGetDataUtil<StudentPaperVo, Long>() {

                @Override
                public List<StudentPaperVo> getData(List<Long> paramList) {
                    return paperService.listByStudentIds(paramList);
                }
            }.getDataForBatch(studentIds, 20);

            if (CollectionUtils.isNotEmpty(paperList)) {
                Map<Long, AnswerPaperVo> paperMap = new HashMap<>();
                for (StudentPaperVo p : paperList) {
                    AnswerQueryVo vo = map.get(p.getStudentId());
                    if (vo == null) {
                        continue;
                    }
                    List<AnswerPaperVo> papers = vo.getPapers();
                    if (papers == null) {
                        continue;
                    }
                    if (papers.size() < p.getNumber()) {
                        continue;
                    }
                    AnswerPaperVo pvo = papers.get(p.getNumber() - 1);
                    pvo.setId(p.getPaperId());
                    pvo.setNumber(p.getNumber());
                    pvo.setAssigned(p.getAssigned());
                    paperMap.put(p.getPaperId(), pvo);
                }
                //查找page
                List<Long> paperIds = paperList.stream().map(p -> p.getPaperId()).collect(Collectors.toList());
                List<PaperPageEntity> paperPageList = new BatchGetDataUtil<PaperPageEntity, Long>() {

                    @Override
                    public List<PaperPageEntity> getData(List<Long> paramList) {
                        return paperPageService.listByPaperList(paramList);
                    }
                }.getDataForBatch(paperIds, 20);

                if (CollectionUtils.isNotEmpty(paperPageList)) {
                    for (PaperPageEntity p : paperPageList) {
                        AnswerPaperVo pvo = paperMap.get(p.getPaperId());
                        if (pvo == null) {
                            continue;
                        }
                        List<AnswerPageVo> pages = pvo.getPages();
                        if (pages == null) {
                            pages = new ArrayList<>();
                            pvo.setPages(pages);
                        }
                        AnswerPageVo pageVo = new AnswerPageVo();
                        pageVo.setIndex(p.getPageIndex());
                        pageVo.setSheetUri(p.getSheetPath());
                        pageVo.setSliceUri(p.getSlicePath());
                        if (query.getWithOmrDetail() != null && query.getWithOmrDetail()) {
                            pageVo.setAbsent(p.getAbsent());
                            pageVo.setBreach(p.getBreach());
                            pageVo.setPaperType(p.getPaperType());
                            pageVo.setQuestion(p.getQuestion());
                            pageVo.setSelective(p.getSelective());
                            pageVo.setRecogData(p.getRecogData());
                        }
                        pages.add(pageVo);
                    }
                }
            }
        }
        return PageUtil.of(iPage);
    }

    @Override
    public List<StudentUploadVo> findToUpload(int pageSize, Long schoolId) {
        if (pageSize <= 0) {
            pageSize = 100;
        }
        return baseMapper.findToUpload(pageSize, schoolId);
    }

    @Override
    public List<AbsentQueryVo> absentExportList(AbsentQueryDomain query) {
        if (query.getExamId() == null) {
            throw new ParameterException("examId 不能为空");
        }
        if (query.getGroupType() == null) {
            throw new ParameterException("groupType 不能为空");
        }
        if (CollectionUtils.isEmpty(query.getStatus())) {
            throw new ParameterException("status 不能为空");
        }
        String groupNameCol = null;
        if (GroupType.PACKAGE.equals(query.getGroupType())) {
            groupNameCol = "package_code";
        }
        if (GroupType.SUBJECT.equals(query.getGroupType())) {
            groupNameCol = "subject_code";
        }
        if (GroupType.CAMPUS.equals(query.getGroupType())) {
            groupNameCol = "campus_name";
        }
        if (GroupType.EXAM_SITE.equals(query.getGroupType())) {
            groupNameCol = "exam_site";
        }
        if (GroupType.EXAM_ROOM.equals(query.getGroupType())) {
            groupNameCol = "exam_room";
        }
        if (GroupType.STUDENT_CODE.equals(query.getGroupType())) {
            groupNameCol = "student_code";
        }
        List<AbsentQueryVo> list = baseMapper.absentQueryList(groupNameCol, query);
        return list;
    }

    @Override
    public List<AnswerExportVo> exportList(AnswerQueryDomain query) {
        if (query.getExamId() == null) {
            throw new ParameterException("examId 不能为空");
        }
        List<AnswerExportVo> list = baseMapper.queryList(query);
        if (CollectionUtils.isNotEmpty(list)) {
            new BatchSetDataUtil<AnswerExportVo>() {

                @Override
                protected void setData(List<AnswerExportVo> dataList) {
                    List<Long> studetnIds = dataList.stream().map(dto -> dto.getId()).distinct()
                            .collect(Collectors.toList());
                    List<StudentPaperCountVo> ret = studentPaperService.findStudentPaperCount(studetnIds);
                    if (ret != null && ret.size() > 0) {
                        Map<Long, Integer> countMap = new HashMap<>();
                        for (StudentPaperCountVo item : ret) {
                            countMap.put(item.getStudentId(), item.getPaperCount());
                        }
                        for (AnswerExportVo vo : list) {
                            vo.setPaperCount(countMap.get(vo.getId()));
                        }
                    }
                }
            }.setDataForBatch(list, 20);
        }
        return list;
    }

    @Transactional
    @Override
    public void updateDataUploadStatus(Long id, UploadStatus status) {
        if (id == null) {
            throw new ParameterException("id 不能为空");
        }
        if (status == null) {
            throw new ParameterException("status 不能为空");
        }
        StudentEntity s = getById(id);
        if (s == null) {
            throw new ParameterException("未找到考生");
        }
        s.setDataUploadStatus(status);
        this.saveOrUpdate(s);
    }

    @Transactional
    @Override
    public void updateFileUploadStatus(Long id, UploadStatus status) {
        if (id == null) {
            throw new ParameterException("id 不能为空");
        }
        if (status == null) {
            throw new ParameterException("status 不能为空");
        }
        StudentEntity s = getById(id);
        if (s == null) {
            throw new ParameterException("未找到考生");
        }
        s.setFileUploadStatus(status);
        this.saveOrUpdate(s);
    }

    @Transactional
    @Override
    public void updateUploadStatus(Long id, UploadStatus fileUploadStatus, UploadStatus dataUploadStatus) {
        if (id == null) {
            throw new ParameterException("id 不能为空");
        }
        if (fileUploadStatus == null) {
            throw new ParameterException("fileUploadStatus 不能为空");
        }
        if (dataUploadStatus == null) {
            throw new ParameterException("dataUploadStatus 不能为空");
        }
        StudentEntity s = getById(id);
        if (s == null) {
            throw new ParameterException("未找到考生");
        }
        s.setFileUploadStatus(fileUploadStatus);
        s.setDataUploadStatus(dataUploadStatus);
        this.saveOrUpdate(s);
    }

    @Transactional
    @Override
    public void updateOmrAbsent(Long id, Boolean omrAbsent) {
        if (id == null) {
            throw new ParameterException("id 不能为空");
        }
        if (omrAbsent == null) {
            throw new ParameterException("omrAbsent 不能为空");
        }
        StudentEntity s = getById(id);
        if (s == null) {
            throw new ParameterException("未找到考生");
        }
        s.setOmrAbsent(omrAbsent);
        this.saveOrUpdate(s);
    }

    @Transactional
    @Override
    public void deltetByExamIdAndUnCreateTime(Long examId, long createTime) {
        QueryWrapper<StudentEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentEntity> lw = wrapper.lambda();
        lw.eq(StudentEntity::getExamId, examId);
        lw.ne(StudentEntity::getCreateTime, createTime);
        this.baseMapper.delete(wrapper);
    }

    @Override
    public boolean existUploadData(Long schoolId) {
        List<StudentUploadVo> list = baseMapper.findToUpload(1, schoolId);
        return CollectionUtils.isNotEmpty(list);
    }

    @Override
    public int getCountByExamAndCardNumber(Long examId, Integer number) {
        QueryWrapper<StudentEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentEntity> lw = wrapper.lambda();
        lw.eq(StudentEntity::getExamId, examId);
        lw.eq(StudentEntity::getCardNumber, number);
        return this.count(wrapper);
    }

    @Transactional
    @Override
    public int importStudent(List<ImportStudentDomain> students) {
        List<StudentEntity> list = new ArrayList<StudentEntity>();
        // Map<Long,Map<String, StudentEntity>> map =new HashMap<Long,
        // Map<String,StudentEntity>>();
        for (ImportStudentDomain domain : students) {
            // Map<String, StudentEntity> examMap =map.get(domain.getExamId());
            // if(examMap ==null) {
            // examMap = new HashMap<String, StudentEntity>();
            // List<StudentEntity> list2 = this.list();
            // for (StudentEntity s : list2) {
            // examMap.put(s.getExamNumber(), s);
            // }
            // map.put(domain.getExamId(), examMap);
            // }
            // StudentEntity entity = examMap.get(domain.getExamNumber());
            StudentEntity entity = this.findByExamAndExamNumber(domain.getExamId(), domain.getExamNumber());
            if (entity == null) {
                entity = this.findByExamAndSubjectCodeAndStudentCode(domain.getExamId(), domain.getSubjectCode(),
                        domain.getStudentCode());
            }
            if (entity == null) {
                entity = new StudentEntity();
                entity.setExamId(domain.getExamId());
                entity.setName(domain.getName());
                entity.setExamNumber(domain.getExamNumber());
                entity.setSubjectCode(domain.getSubjectCode());
                entity.setStudentCode(domain.getStudentCode());
                entity.setPackageCode(domain.getPackageCode());
                entity.setExamRoom(domain.getExamRoom());
                entity.setExamSite(domain.getExamSite());
                entity.setSeatNumber(domain.getSeatNumber());
                entity.setCampusName(domain.getCampusName());
                entity.setAbsentSuspect(false);
                entity.setAssigned(false);
                entity.setIncomplete(false);
                entity.setOmrAbsent(false);
                entity.setQuestionFilled(false);
                entity.setPaperType("#");
                entity.setStatus(ScanStatus.UNEXIST);
                list.add(entity);
            }
        }
        this.saveBatch(list);
        return list.size();
    }

    @Override
    public StudentEntity findByExamAndSubjectCodeAndStudentCode(Long examId, String subjectCode, String studentCode) {
        if (examId == null) {
            throw new ParameterException("examId 不能为空");
        }
        if (StringUtils.isBlank(subjectCode)) {
            throw new ParameterException("subjectCode不能为空");
        }
        if (StringUtils.isBlank(studentCode)) {
            throw new ParameterException("studentCode不能为空");
        }
        QueryWrapper<StudentEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(StudentEntity::getExamId, examId);
        queryWrapper.lambda().eq(StudentEntity::getSubjectCode, subjectCode);
        queryWrapper.lambda().eq(StudentEntity::getStudentCode, studentCode);
        return baseMapper.selectOne(queryWrapper);
    }

    @Override
    public int countByQuery(ImportStudentQueryVo query) {
        if (query.getExamId() == null) {
            throw new ParameterException("examId不能为空");
        }
        return baseMapper.countByQuery(query);
    }

    @Override
    public List<ImportStudentVo> findByQuery(ImportStudentQueryVo query) {
        if (query.getExamId() == null) {
            throw new ParameterException("examId不能为空");
        }
        ExamEntity exam = examService.getById(query.getExamId());
        if (exam == null) {
            throw new ParameterException("未找到考试");
        }
        IPage<ImportStudentVo> iPage = baseMapper
                .listPageQuery(new Page<ImportStudentVo>(query.getPageNumber(), query.getPageSize()), query);
        if (query.getWithAnswer() != null && query.getWithAnswer()) {
            for (ImportStudentVo student : iPage.getRecords()) {
                List<StudentPaperEntity> papers = studentPaperService.findByStudentId(student.getId());
                List<PaperPageEntity> pageList = new ArrayList<>();
                for (StudentPaperEntity sp : papers) {
                    List<PaperPageEntity> pages = paperPageService.listByPaperId(sp.getPaperId());
                    pageList.addAll(pages);
                }
                List<String> answers = new ArrayList<String>();
                for (PaperPageEntity p : pageList) {
                    if (p.getPageIndex() == 1) {
                        student.setPaperType(p.getPaperType() == null ? null : p.getPaperType().getResult());
                    }
                    if (p.getQuestion() != null && CollectionUtils.isNotEmpty(p.getQuestion().getResult())) {
                        answers.addAll(p.getQuestion().getResult());
                    }
                }
                student.setAnswer(answers);
            }
        }
        return iPage.getRecords();
    }

    @Override
    public int getOmrAbsentCountByExam(Long examId) {
        QueryWrapper<StudentEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentEntity> lw = wrapper.lambda();
        lw.eq(StudentEntity::getExamId, examId);
        lw.eq(StudentEntity::getOmrAbsent, true);
        return this.count(wrapper);
    }

    @Override
    public boolean existPictureCopyData() {
        List<StudentUploadVo> list = baseMapper.findToPictureCopy(1);
        return CollectionUtils.isNotEmpty(list);
    }

    @Override
    public List<StudentUploadVo> findToPictureCopy(int pageSize) {
        if (pageSize <= 0) {
            pageSize = 100;
        }
        return baseMapper.findToPictureCopy(pageSize);
    }

    @Override
    public Double getUploadProgress(Long examId) {
        int total = getCountByExam(examId);
        if (total == 0) {
            return 0.0;
        }
        int uploaded = baseMapper.getUploadedCount(examId);
        BigDecimal totalB = new BigDecimal(total);
        BigDecimal uploadedB = new BigDecimal(uploaded);
        return uploadedB.divide(totalB, 2, BigDecimal.ROUND_HALF_UP).doubleValue();
    }

    @Transactional
    @Override
    public void studentClean(Long examId) {
        ExamEntity exam = examService.getById(examId);
        if (exam == null) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        if (batchService.getCountByExam(examId) > 0) {
            throw new ParameterException("已开始扫描不能清空");
        }
        QueryWrapper<StudentEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentEntity> lw = wrapper.lambda();
        lw.eq(StudentEntity::getExamId, examId);
        this.baseMapper.delete(wrapper);
        subjectService.cleanByExamId(examId);
    }
}
