package cn.com.qmth.scancloud.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.jeffreyning.mybatisplus.service.MppServiceImpl;

import cn.com.qmth.scancloud.bean.ImportSubjectDomain;
import cn.com.qmth.scancloud.dao.SubjectDao;
import cn.com.qmth.scancloud.entity.SubjectEntity;
import cn.com.qmth.scancloud.service.SubjectService;

@Service
public class SubjectServiceImpl extends MppServiceImpl<SubjectDao, SubjectEntity> implements SubjectService {

	@Transactional
	@Override
	public int cleanByExamId(Long examId) {
		QueryWrapper<SubjectEntity> wrapper = new QueryWrapper<>();
		LambdaQueryWrapper<SubjectEntity> lw = wrapper.lambda();
		lw.eq(SubjectEntity::getExamId, examId);
		return this.baseMapper.delete(wrapper);
	}

	@Override
	public List<SubjectEntity> listByExamId(Long examId) {
		QueryWrapper<SubjectEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(SubjectEntity::getExamId, examId);
		queryWrapper.lambda().orderByDesc(SubjectEntity::getCode);
		return this.list(queryWrapper);
	}

	@Override
	public SubjectEntity findByExamIdAndCode(Long examId, String code) {
		QueryWrapper<SubjectEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(SubjectEntity::getExamId, examId);
		queryWrapper.lambda().eq(SubjectEntity::getCode, code);
		return this.getOne(queryWrapper);
	}

	@Override
	public int countByExamId(Long examId) {
		QueryWrapper<SubjectEntity> wrapper = new QueryWrapper<>();
		LambdaQueryWrapper<SubjectEntity> lw = wrapper.lambda();
		lw.eq(SubjectEntity::getExamId, examId);
		return this.count(wrapper);
	}

	@Transactional
	@Override
	public int importSubject(List<ImportSubjectDomain> subjects) {
		Map<Long, Set<String>> map = new HashMap<Long, Set<String>>();
		List<SubjectEntity> list = this.list();
		for (SubjectEntity subjectEntity : list) {
			Set<String> set = map.get(subjectEntity.getExamId());
			if (set == null) {
				set = new HashSet<String>();
			}
			set.add(subjectEntity.getCode());
			map.put(subjectEntity.getExamId(), set);
		}

		List<SubjectEntity> saveList = new ArrayList<SubjectEntity>();
		for (ImportSubjectDomain importSubjectDomain : subjects) {
			Set<String> set = map.get(importSubjectDomain.getExamId());
			if (set != null) {
				if (!set.contains(importSubjectDomain.getSubjectCode())) {
					SubjectEntity subjectEntity = new SubjectEntity(importSubjectDomain.getExamId(),
							importSubjectDomain.getSubjectCode(), importSubjectDomain.getSubjectName());
					set.add(importSubjectDomain.getSubjectCode());
					saveList.add(subjectEntity);
				}
			} else {
				set = new HashSet<String>();
				SubjectEntity subjectEntity = new SubjectEntity(importSubjectDomain.getExamId(),
						importSubjectDomain.getSubjectCode(), importSubjectDomain.getSubjectName());
				set.add(importSubjectDomain.getSubjectCode());
				saveList.add(subjectEntity);
			}
			map.put(importSubjectDomain.getExamId(), set);
		}
		this.saveOrUpdateBatchByMultiId(saveList);
		return saveList.size();
	}

}
