package cn.com.qmth.scancloud.service.impl;

import java.io.File;
import java.util.List;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.qmth.boot.core.exception.ParameterException;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.boot.core.fss.store.FileStore;

import cn.com.qmth.scancloud.config.SysProperty;
import cn.com.qmth.scancloud.entity.PaperPageEntity;
import cn.com.qmth.scancloud.entity.StudentEntity;
import cn.com.qmth.scancloud.entity.StudentPaperEntity;
import cn.com.qmth.scancloud.entity.SystemConfigEntity;
import cn.com.qmth.scancloud.enums.ImageTransferMode;
import cn.com.qmth.scancloud.enums.SystemMode;
import cn.com.qmth.scancloud.service.PaperPageService;
import cn.com.qmth.scancloud.service.StudentPaperService;
import cn.com.qmth.scancloud.service.StudentService;
import cn.com.qmth.scancloud.service.SystemConfigService;
import cn.com.qmth.scancloud.service.ToolExportService;

@Service
public class ToolExportServiceImpl implements ToolExportService {

	@Autowired
	private FileStore fileStore;
	@Autowired
	private PaperPageService paperPageService;
	@Autowired
	private StudentPaperService studentPaperService;

	@Autowired
	private StudentService studentService;

	@Autowired
	private SystemConfigService systemConfigService;

	@Autowired
	private SysProperty sysProperty;

	private String getOwSheetUri(Long examId, String subjectCode, String examNumber, Integer paperNumber,
			Integer pageIndex) {
		return String.format("%s/%s/%s/%s/%s/%s-%s.jpg", "sheet", examId, subjectCode, examNumber.substring(0, 4),
				examNumber, paperNumber, pageIndex);
	}

	private String getOwSliceUri(Long examId, String subjectCode, String examNumber, Integer paperNumber,
			Integer pageIndex, Integer index) {
		return String.format("%s/%s/%s/%s/%s/%s-%s-%s.jpg", "slice", examId, subjectCode, examNumber.substring(0, 4),
				examNumber, paperNumber, pageIndex, index);
	}

	private String getCetSheetUri(Long examId, String subjectCode, String examNumber, Integer paperNumber,
			Integer pageIndex) {
		return String.format("%s/%s/%s/%s/%s/%s-%s.jpg", "sheet", examId, subjectCode, examNumber.substring(0, 4),
				examNumber, paperNumber, pageIndex);
	}

	private String getCetSliceUri(Long examId, String subjectCode, String examNumber, Integer paperNumber,
			Integer pageIndex, Integer index) {
		return String.format("%s/%s/%s/%s/%s/%s-%s-%s.jpg", "slice", examId, subjectCode, examNumber.substring(0, 4),
				examNumber, paperNumber, pageIndex, index);
	}

	@Override
	public void studentCheck(Long studentId) {
		StudentEntity student = studentService.getById(studentId);
		if (student == null) {
			throw new ParameterException("未找到考生");
		}
		SystemConfigEntity config = systemConfigService.findConfig();
		if (config == null) {
			throw new ParameterException("未找到系统配置");
		}
		if (!SystemMode.STANDALONE.equals(config.getSystemMode())) {
			throw new ParameterException("系统配置非独立模式");
		}
		if (config.getImageTransferMode() == null) {
			throw new ParameterException("系统配置转存模式为空");
		}
		List<StudentPaperEntity> spes = studentPaperService.findByStudentId(studentId);
		if (CollectionUtils.isEmpty(spes)) {
			throw new ParameterException("没有扫描结果");
		}
		String standAloneDir = sysProperty.getTransferDir();
		for (StudentPaperEntity spe : spes) {
			List<PaperPageEntity> pgs = paperPageService.listByPaperId(spe.getPaperId());
			if (CollectionUtils.isEmpty(pgs)) {
				throw new ParameterException("没有扫描页信息");
			}
			for (PaperPageEntity pg : pgs) {
				String sheetPath = pg.getSheetPath();
				if (StringUtils.isBlank(sheetPath)) {
					throw new ParameterException("没有原图");
				}
				String mirrorSheetPath = null;
				if (ImageTransferMode.CET.equals(config.getImageTransferMode())) {
					mirrorSheetPath = getCetSheetUri(student.getExamId(), student.getSubjectCode(),
							student.getExamNumber(), spe.getPaperNumber(), pg.getPageIndex());
				} else if (ImageTransferMode.OW.equals(config.getImageTransferMode())) {
					mirrorSheetPath = getOwSheetUri(student.getExamId(), student.getSubjectCode(),
							student.getExamNumber(), spe.getPaperNumber(), pg.getPageIndex());
				} else {
					throw new ParameterException("转存模式配置错误");
				}
				File targetSheetFile = new File(standAloneDir+"/"+mirrorSheetPath);
				if(!targetSheetFile.exists()) {
					try {
						FileUtils.copyToFile(fileStore.read(sheetPath), targetSheetFile);
					} catch (Exception e) {
						throw new StatusException("文件拷贝出错",e);
					}
				}
				List<String> slices = pg.getSlicePath();
				if (CollectionUtils.isNotEmpty(slices)) {
					int index=0;
					for(String slice:slices) {
						index++;
						String mirrorSlicePath = null;
						if (ImageTransferMode.CET.equals(config.getImageTransferMode())) {
							mirrorSlicePath = getCetSliceUri(student.getExamId(), student.getSubjectCode(),
									student.getExamNumber(), spe.getPaperNumber(), pg.getPageIndex(),index);
						} else if (ImageTransferMode.OW.equals(config.getImageTransferMode())) {
							mirrorSlicePath = getOwSliceUri(student.getExamId(), student.getSubjectCode(),
									student.getExamNumber(), spe.getPaperNumber(), pg.getPageIndex(),index);
						} else {
							throw new ParameterException("转存模式配置错误");
						}
						File targetSliceFile = new File(standAloneDir+"/"+mirrorSlicePath);
						if(!targetSliceFile.exists()) {
							try {
								FileUtils.copyToFile(fileStore.read(slice), targetSliceFile);
							} catch (Exception e) {
								throw new StatusException("文件拷贝出错",e);
							}
						}
					}
				}
			}
		}
	}
	
	@Override
	public void studentFileCopy(Long studentId) {
		StudentEntity student = studentService.getById(studentId);
		if (student == null) {
			throw new ParameterException("未找到考生");
		}
		SystemConfigEntity config = systemConfigService.findConfig();
		if (config == null) {
			throw new ParameterException("未找到系统配置");
		}
		if (!SystemMode.STANDALONE.equals(config.getSystemMode())) {
			return;
		}
		if (config.getImageTransferMode() == null) {
			throw new ParameterException("系统配置转存模式为空");
		}
		List<StudentPaperEntity> spes = studentPaperService.findByStudentId(studentId);
		if (CollectionUtils.isEmpty(spes)) {
			return;
		}
		String standAloneDir = sysProperty.getTransferDir();
		for (StudentPaperEntity spe : spes) {
			List<PaperPageEntity> pgs = paperPageService.listByPaperId(spe.getPaperId());
			if (CollectionUtils.isEmpty(pgs)) {
				throw new ParameterException("没有扫描页信息");
			}
			for (PaperPageEntity pg : pgs) {
				String sheetPath = pg.getSheetPath();
				if (StringUtils.isBlank(sheetPath)) {
					throw new ParameterException("没有原图");
				}
				String mirrorSheetPath = null;
				if (ImageTransferMode.CET.equals(config.getImageTransferMode())) {
					mirrorSheetPath = getCetSheetUri(student.getExamId(), student.getSubjectCode(),
							student.getExamNumber(), spe.getPaperNumber(), pg.getPageIndex());
				} else if (ImageTransferMode.OW.equals(config.getImageTransferMode())) {
					mirrorSheetPath = getOwSheetUri(student.getExamId(), student.getSubjectCode(),
							student.getExamNumber(), spe.getPaperNumber(), pg.getPageIndex());
				} else {
					throw new ParameterException("转存模式配置错误");
				}
				File targetSheetFile = new File(standAloneDir+"/"+mirrorSheetPath);
				if(targetSheetFile.exists()) {
					targetSheetFile.delete();
				}
				try {
					FileUtils.copyToFile(fileStore.read(sheetPath), targetSheetFile);
				} catch (Exception e) {
					throw new StatusException("文件拷贝出错",e);
				}
				List<String> slices = pg.getSlicePath();
				if (CollectionUtils.isNotEmpty(slices)) {
					int index=0;
					for(String slice:slices) {
						index++;
						String mirrorSlicePath = null;
						if (ImageTransferMode.CET.equals(config.getImageTransferMode())) {
							mirrorSlicePath = getCetSliceUri(student.getExamId(), student.getSubjectCode(),
									student.getExamNumber(), spe.getPaperNumber(), pg.getPageIndex(),index);
						} else if (ImageTransferMode.OW.equals(config.getImageTransferMode())) {
							mirrorSlicePath = getOwSliceUri(student.getExamId(), student.getSubjectCode(),
									student.getExamNumber(), spe.getPaperNumber(), pg.getPageIndex(),index);
						} else {
							throw new ParameterException("转存模式配置错误");
						}
						File targetSliceFile = new File(standAloneDir+"/"+mirrorSlicePath);
						if(targetSliceFile.exists()) {
							targetSliceFile.delete();
						}
						try {
							FileUtils.copyToFile(fileStore.read(slice), targetSliceFile);
						} catch (Exception e) {
							throw new StatusException("文件拷贝出错",e);
						}
					}
				}
			}
		}
	}

}
