package cn.com.qmth.scancloud.task;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.boot.core.exception.StatusException;

import cn.com.qmth.scancloud.bean.User;
import cn.com.qmth.scancloud.enums.LockType;
import cn.com.qmth.scancloud.service.ExamService;

public class ExamStudentSyncThread implements Runnable {

    protected static Logger log = LoggerFactory.getLogger(ExamStudentSyncThread.class);

    private User user;

    private Long examId;

    private ExamService examService;

    private ConcurrentService concurrentService;

    public ExamStudentSyncThread(User user, Long examId, ExamService examService, ConcurrentService concurrentService) {
        this.examId = examId;
        this.user = user;
        this.examService = examService;
        this.concurrentService = concurrentService;
    }

    @Override
    public void run() {
        if (examId == null) {
            return;
        }
        if (user == null) {
            return;
        }
        try {
            if(concurrentService.getLock(LockType.EXAM_SYNC + "-" + examId).tryLock()) {
                examService.sync(user, examId);
            }
        } catch (Exception e) {
            log.error("exam student sync error", e);
            throw new StatusException("考生同步失败");
        } finally {
            concurrentService.getLock(LockType.EXAM_SYNC + "-" + examId).unlock();
        }
    }

}
